@@ -479,7 +479,7 @@ static HRESULT file_register(Registrar *This, LPCOLESTR fileName, BOOL do_regist
     HRESULT hres;
 
     file = CreateFileW(fileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_READONLY, NULL);
-    if(file) {
+    if(file != INVALID_HANDLE_VALUE) {
         filelen = GetFileSize(file, NULL);
         regstra = HeapAlloc(GetProcessHeap(), 0, filelen);
         lres = ReadFile(file, regstra, filelen, NULL, NULL);
@@ -756,20 +756,27 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID riid, LPVOID *ppvObject)
 
 extern HINSTANCE hInst;
 
-static HRESULT do_register_dll_server(LPCOLESTR wszDll, LPCOLESTR wszId, BOOL do_register)
+static HRESULT do_register_dll_server(IRegistrar *pRegistrar, LPCOLESTR wszDll,
+                                      LPCOLESTR wszId, BOOL do_register,
+                                      const struct _ATL_REGMAP_ENTRY* pMapEntries)
 {
     WCHAR buf[MAX_PATH];
     HRESULT hres;
-    IRegistrar *pRegistrar;
+    const struct _ATL_REGMAP_ENTRY *pMapEntry;
 
     static const WCHAR wszModule[] = {'M','O','D','U','L','E',0};
     static const WCHAR wszRegistry[] = {'R','E','G','I','S','T','R','Y',0};
     static const WCHAR wszCLSID_ATLRegistrar[] =
             {'C','L','S','I','D','_','A','T','L','R','e','g','i','s','t','r','a','r',0};
 
-    Registrar_create(NULL, &IID_IRegistrar, (void**)&pRegistrar);
+    if (!pRegistrar)
+        Registrar_create(NULL, &IID_IRegistrar, (void**)&pRegistrar);
+
     IRegistrar_AddReplacement(pRegistrar, wszModule, wszDll);
 
+    for (pMapEntry = pMapEntries; pMapEntry && pMapEntry->szKey; pMapEntry++)
+        IRegistrar_AddReplacement(pRegistrar, pMapEntry->szKey, pMapEntry->szData);
+
     StringFromGUID2(&CLSID_ATLRegistrar, buf, sizeof(buf)/sizeof(buf[0]));
     IRegistrar_AddReplacement(pRegistrar, wszCLSID_ATLRegistrar, buf);
 
@@ -785,7 +792,7 @@ static HRESULT do_register_dll_server(LPCOLESTR wszDll, LPCOLESTR wszId, BOOL do
 static HRESULT do_register_server(BOOL do_register)
 {
     static const WCHAR wszDll[] = {'a','t','l','.','d','l','l',0};
-    return do_register_dll_server(wszDll, MAKEINTRESOURCEW(101), do_register);
+    return do_register_dll_server(NULL, wszDll, MAKEINTRESOURCEW(101), do_register, NULL);
 }
 
 /***********************************************************************
@@ -801,11 +808,6 @@ HRESULT WINAPI AtlModuleUpdateRegistryFromResourceD(_ATL_MODULEW* pM, LPCOLESTR
      */
     WCHAR module_name[MAX_PATH];
 
-    if(pMapEntries || pReg) {
-        FIXME("MapEntries and Registrar parameter not supported\n");
-        return E_FAIL;
-    }
-
     if(!GetModuleFileNameW(lhInst, module_name, MAX_PATH)) {
         FIXME("hinst %p: did not get module name\n",
         lhInst);
@@ -815,7 +817,7 @@ HRESULT WINAPI AtlModuleUpdateRegistryFromResourceD(_ATL_MODULEW* pM, LPCOLESTR
     TRACE("%p (%s), %s, %d, %p, %p\n", hInst, debugstr_w(module_name),
 	debugstr_w(lpszRes), bRegister, pMapEntries, pReg);
 
-    return do_register_dll_server(module_name, lpszRes, bRegister);
+    return do_register_dll_server(pReg, module_name, lpszRes, bRegister, pMapEntries);
 }
 
 /***********************************************************************
