@@ -73,7 +73,7 @@ typedef	struct
     int		rate;
 } Format;
 
-static Format PCM_Formats[] =
+static const Format PCM_Formats[] =
 {
     {1,  8,  8000}, {2,  8,  8000}, {1, 16,  8000}, {2, 16,  8000},
     {1,  8, 11025}, {2,  8, 11025}, {1, 16, 11025}, {2, 16, 11025},
@@ -81,7 +81,7 @@ static Format PCM_Formats[] =
     {1,  8, 44100}, {2,  8, 44100}, {1, 16, 44100}, {2, 16, 44100},
 };
 
-static Format ADPCM_Formats[] =
+static const Format ADPCM_Formats[] =
 {
     {1,  4,  8000}, {2,	4,  8000},  {1,  4, 11025}, {2,	 4, 11025},
     {1,  4, 22050}, {2,	4, 22050},  {1,  4, 44100}, {2,	 4, 44100},
@@ -95,8 +95,8 @@ static Format ADPCM_Formats[] =
  */
 static	DWORD	ADPCM_GetFormatIndex(LPWAVEFORMATEX wfx)
 {
-    int 	i, hi;
-    Format*	fmts;
+    int             i, hi;
+    const Format*   fmts;
 
     switch (wfx->wFormatTag)
     {
@@ -523,7 +523,7 @@ static	LRESULT	ADPCM_FormatTagDetails(PACMFORMATTAGDETAILSW aftd, DWORD dwQuery)
 	}
 	break;
     default:
-	WARN("Unsupported query %08lx\n", dwQuery);
+	WARN("Unsupported query %08x\n", dwQuery);
 	return MMSYSERR_NOTSUPPORTED;
     }
 
@@ -589,12 +589,12 @@ static	LRESULT	ADPCM_FormatDetails(PACMFORMATDETAILSW afd, DWORD dwQuery)
                 ((IMAADPCMWAVEFORMAT*)afd->pwfx)->wSamplesPerBlock = (1024 - 4 * afd->pwfx->nChannels) * (2 / afd->pwfx->nChannels) + 1;
 	    break;
 	default:
-	    WARN("Unsupported tag %08lx\n", afd->dwFormatTag);
+            WARN("Unsupported tag %08x\n", afd->dwFormatTag);
 	    return MMSYSERR_INVALPARAM;
 	}
 	break;
     default:
-	WARN("Unsupported query %08lx\n", dwQuery);
+	WARN("Unsupported query %08x\n", dwQuery);
 	return MMSYSERR_NOTSUPPORTED;
     }
     afd->fdwSupport = ACMDRIVERDETAILS_SUPPORTF_CODEC;
@@ -769,17 +769,6 @@ static	LRESULT	ADPCM_StreamClose(PACMDRVSTREAMINSTANCE adsi)
     return MMSYSERR_NOERROR;
 }
 
-/***********************************************************************
- *           ADPCM_round
- *
- */
-static	inline DWORD	ADPCM_round(DWORD a, DWORD b, DWORD c)
-{
-    assert(a && b && c);
-    /* to be sure, always return an entire number of c... */
-    return ((double)a * (double)b + (double)c - 1) / (double)c;
-}
-
 /***********************************************************************
  *           ADPCM_StreamSize
  *
@@ -827,7 +816,7 @@ static	LRESULT ADPCM_StreamSize(PACMDRVSTREAMINSTANCE adsi, PACMDRVSTREAMSIZE ad
 	}
 	break;
     default:
-	WARN("Unsupported query %08lx\n", adss->fdwSize);
+	WARN("Unsupported query %08x\n", adss->fdwSize);
 	return MMSYSERR_NOTSUPPORTED;
     }
     return MMSYSERR_NOERROR;
@@ -848,7 +837,7 @@ static LRESULT ADPCM_StreamConvert(PACMDRVSTREAMINSTANCE adsi, PACMDRVSTREAMHEAD
 	  ACM_STREAMCONVERTF_END|
 	  ACM_STREAMCONVERTF_START))
     {
-	FIXME("Unsupported fdwConvert (%08lx), ignoring it\n", adsh->fdwConvert);
+	FIXME("Unsupported fdwConvert (%08x), ignoring it\n", adsh->fdwConvert);
     }
     /* ACM_STREAMCONVERTF_BLOCKALIGN
      *	currently all conversions are block aligned, so do nothing for this flag
