@@ -45,7 +45,7 @@ WINE_DECLARE_DEBUG_CHANNEL(io);
 #define ADD_LOWORD(dw,val)  ((dw) = ((dw) & 0xffff0000) | LOWORD((DWORD)(dw)+(val)))
 #define ISV86(context)      ((context)->EFlags & 0x00020000)
 
-inline static void add_stack( CONTEXT86 *context, int offset )
+static inline void add_stack( CONTEXT86 *context, int offset )
 {
     if (ISV86(context) || !IS_SELECTOR_32BIT(context->SegSs))
         ADD_LOWORD( context->Esp, offset );
@@ -53,7 +53,7 @@ inline static void add_stack( CONTEXT86 *context, int offset )
         context->Esp += offset;
 }
 
-inline static void *make_ptr( CONTEXT86 *context, DWORD seg, DWORD off, int long_addr )
+static inline void *make_ptr( CONTEXT86 *context, DWORD seg, DWORD off, int long_addr )
 {
     if (ISV86(context)) return (void *)((seg << 4) + LOWORD(off));
     if (wine_ldt_is_system(seg)) return (void *)off;
@@ -61,7 +61,7 @@ inline static void *make_ptr( CONTEXT86 *context, DWORD seg, DWORD off, int long
     return (char *) MapSL( MAKESEGPTR( seg, 0 ) ) + off;
 }
 
-inline static void *get_stack( CONTEXT86 *context )
+static inline void *get_stack( CONTEXT86 *context )
 {
     if (ISV86(context)) return (void *)((context->SegSs << 4) + LOWORD(context->Esp));
     return wine_ldt_get_ptr( context->SegSs, context->Esp );
@@ -77,7 +77,7 @@ struct idtr
 
 static LDT_ENTRY idt[256];
 
-inline static struct idtr get_idtr(void)
+static inline struct idtr get_idtr(void)
 {
     struct idtr ret;
 #if defined(__i386__) && defined(__GNUC__)
