@@ -65,10 +65,16 @@ static BOOL  (WINAPI *pAddPortUI)(PCWSTR, HWND, PCWSTR, PWSTR *);
 static BOOL  (WINAPI *pConfigurePortUI)(PCWSTR, HWND, PCWSTR);
 static BOOL  (WINAPI *pDeletePortUI)(PCWSTR, HWND, PCWSTR);
 
+static HANDLE hXcv;
+static HANDLE hXcv_noaccess;
 
+/* ########################### */
+
+static const WCHAR cmd_AddPortW[] = {'A','d','d','P','o','r','t',0};
 static const WCHAR cmd_ConfigureLPTPortCommandOKW[] = {'C','o','n','f','i','g','u','r','e',
                                     'L','P','T','P','o','r','t',
                                     'C','o','m','m','a','n','d','O','K',0};
+static WCHAR cmd_DeletePortW[] = {'D','e','l','e','t','e','P','o','r','t',0};
 static WCHAR cmd_GetTransmissionRetryTimeoutW[] = {'G','e','t',
                                     'T','r','a','n','s','m','i','s','s','i','o','n',
                                     'R','e','t','r','y','T','i','m','e','o','u','t',0};
@@ -343,9 +349,8 @@ static void test_InitializePrintMonitor(void)
 static void test_XcvClosePort(void)
 {
     DWORD   res;
-    HANDLE hXcv;
+    HANDLE hXcv2;
 
-    if ((pXcvOpenPort == NULL) || (pXcvClosePort == NULL)) return;
 
     if (0)
     {
@@ -356,53 +361,88 @@ static void test_XcvClosePort(void)
 
 
     SetLastError(0xdeadbeef);
-    hXcv = (HANDLE) 0xdeadbeef;
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
+    hXcv2 = (HANDLE) 0xdeadbeef;
+    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv2);
+    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv2);
 
     if (res) {
         SetLastError(0xdeadbeef);
-        res = pXcvClosePort(hXcv);
+        res = pXcvClosePort(hXcv2);
         ok( res, "returned %d with %u (expected '!= 0')\n", res, GetLastError());
 
         if (0)
         {
         /* test for "Double Free": crash with native localspl.dll (w2k+xp) */
-        res = pXcvClosePort(hXcv);
+        res = pXcvClosePort(hXcv2);
         }
     }
 }
 
 /* ########################### */
 
+static void test_XcvDataPort_AddPort(void)
+{
+    DWORD   res;
+
+
+    /*
+     * The following tests crash with native localspl.dll on w2k and xp,
+     * but it works, when the native dll (w2k and xp) is used in wine.
+     * also tested (same crash): replacing emptyW with portname_lpt1W
+     * and replacing "NULL, 0, NULL" with "buffer, MAX_PATH, &needed"
+     *
+     * We need to use a different API (AddPortEx) instead
+     */
+    if (0)
+    {
+    /* create a Port for a normal, writable file */
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_AddPortW, (PBYTE) tempfileW, (lstrlenW(tempfileW) + 1) * sizeof(WCHAR), NULL, 0, NULL);
+
+    /* add our testport again */
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_AddPortW, (PBYTE) tempfileW, (lstrlenW(tempfileW) + 1) * sizeof(WCHAR), NULL, 0, NULL);
+
+    /* create a well-known Port  */
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_AddPortW, (PBYTE) portname_lpt1W, (lstrlenW(portname_lpt1W) + 1) * sizeof(WCHAR), NULL, 0, NULL);
+
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_AddPortW, (PBYTE) portname_lpt1W, (lstrlenW(portname_lpt1W) + 1) * sizeof(WCHAR), NULL, 0, NULL);
+    /* native localspl.dll on wine: ERROR_ALREADY_EXISTS */
+
+    /* ERROR_ALREADY_EXISTS is also returned from native localspl.dll on wine,
+       when "RPT1:" was already installed for redmonnt.dll:
+       res = pXcvDataPort(hXcv, cmd_AddPortW, (PBYTE) portname_rpt1W, ...
+    */
+
+    /* cleanup */
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_DeletePortW, (PBYTE) tempfileW, (lstrlenW(tempfileW) + 1) * sizeof(WCHAR), NULL, 0, NULL);
+    }
+
+}
+
+/* ########################### */
+
 static void test_XcvDataPort_ConfigureLPTPortCommandOK(void)
 {
     CHAR    org_value[16];
     CHAR    buffer[16];
     HKEY    hroot = NULL;
-    HANDLE  hXcv;
     DWORD   res;
     DWORD   needed;
 
-    if ((pXcvOpenPort == NULL) || (pXcvDataPort == NULL) || (pXcvClosePort == NULL)) return;
-
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "hXcv: %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
 
     /* Read the original value from the registry */
     res = RegOpenKeyExA(HKEY_LOCAL_MACHINE, WinNT_CV_WindowsA, 0, KEY_ALL_ACCESS, &hroot);
     if (res == ERROR_ACCESS_DENIED) {
-        pXcvClosePort(hXcv);
         skip("ACCESS_DENIED\n");
         return;
     }
 
     if (res != ERROR_SUCCESS) {
         /* unable to open the registry: skip the test */
-        pXcvClosePort(hXcv);
         skip("got %d\n", res);
         return;
     }
@@ -420,7 +460,6 @@ static void test_XcvDataPort_ConfigureLPTPortCommandOK(void)
     SetLastError(0xdeadbeef);
     res = pXcvDataPort(hXcv, cmd_ConfigureLPTPortCommandOKW, (PBYTE) num_0W, sizeof(num_0W), NULL, 0, &needed);
     if (res == ERROR_INVALID_PARAMETER) {
-        pXcvClosePort(hXcv);
         skip("'ConfigureLPTPortCommandOK' not supported\n");
         return;
     }
@@ -477,7 +516,38 @@ static void test_XcvDataPort_ConfigureLPTPortCommandOK(void)
     }
 
     RegCloseKey(hroot);
-    pXcvClosePort(hXcv);
+
+}
+
+/* ########################### */
+
+static void test_XcvDataPort_DeletePort(void)
+{
+    DWORD   res;
+    DWORD   needed;
+
+
+    /* cleanup: just to make sure */
+    needed = (DWORD) 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_DeletePortW, (PBYTE) tempfileW, (lstrlenW(tempfileW) + 1) * sizeof(WCHAR), NULL, 0, &needed);
+    ok( !res  || (res == ERROR_FILE_NOT_FOUND),
+        "returned %d with %u (expected ERROR_SUCCESS or ERROR_FILE_NOT_FOUND)\n",
+        res, GetLastError());
+
+
+    /* ToDo: cmd_AddPortW for tempfileW, then cmd_DeletePortW for the existing Port */
+
+
+    /* try to delete a non-existing Port */
+    needed = (DWORD) 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    res = pXcvDataPort(hXcv, cmd_DeletePortW, (PBYTE) tempfileW, (lstrlenW(tempfileW) + 1) * sizeof(WCHAR), NULL, 0, &needed);
+    ok( res == ERROR_FILE_NOT_FOUND,
+        "returned %d with %u (expected ERROR_FILE_NOT_FOUND)\n", res, GetLastError());
+
+    /* emptyW as Portname: ERROR_FILE_NOT_FOUND is returned */
+    /* NULL as Portname: Native localspl.dll crashed */
 
 }
 
@@ -487,27 +557,17 @@ static void test_XcvDataPort_GetTransmissionRetryTimeout(void)
 {
     CHAR    org_value[16];
     HKEY    hroot = NULL;
-    HANDLE  hXcv;
     DWORD   buffer[2];
     DWORD   res;
     DWORD   needed;
     DWORD   len;
 
 
-    if ((pXcvOpenPort == NULL) || (pXcvDataPort == NULL) || (pXcvClosePort == NULL)) return;
-
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "hXcv: %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
-
     /* ask for needed size */
     needed = (DWORD) 0xdeadbeef;
     SetLastError(0xdeadbeef);
     res = pXcvDataPort(hXcv, cmd_GetTransmissionRetryTimeoutW, NULL, 0, NULL, 0, &needed);
     if (res == ERROR_INVALID_PARAMETER) {
-        pXcvClosePort(hXcv);
         skip("'GetTransmissionRetryTimeout' not supported\n");
         return;
     }
@@ -520,14 +580,12 @@ static void test_XcvDataPort_GetTransmissionRetryTimeout(void)
     /* Read the original value from the registry */
     res = RegOpenKeyExA(HKEY_LOCAL_MACHINE, WinNT_CV_WindowsA, 0, KEY_ALL_ACCESS, &hroot);
     if (res == ERROR_ACCESS_DENIED) {
-        pXcvClosePort(hXcv);
         skip("ACCESS_DENIED\n");
         return;
     }
 
     if (res != ERROR_SUCCESS) {
         /* unable to open the registry: skip the test */
-        pXcvClosePort(hXcv);
         skip("got %d\n", res);
         return;
     }
@@ -606,7 +664,6 @@ static void test_XcvDataPort_GetTransmissionRetryTimeout(void)
     }
 
     RegCloseKey(hroot);
-    pXcvClosePort(hXcv);
 }
 
 /* ########################### */
@@ -614,26 +671,16 @@ static void test_XcvDataPort_GetTransmissionRetryTimeout(void)
 static void test_XcvDataPort_MonitorUI(void)
 {
     DWORD   res;
-    HANDLE  hXcv;
     BYTE    buffer[MAX_PATH + 2];
     DWORD   needed;
     DWORD   len;
 
 
-    if ((pXcvOpenPort == NULL) || (pXcvDataPort == NULL) || (pXcvClosePort == NULL)) return;
-
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
-
     /* ask for needed size */
     needed = (DWORD) 0xdeadbeef;
     SetLastError(0xdeadbeef);
     res = pXcvDataPort(hXcv, cmd_MonitorUIW, NULL, 0, NULL, 0, &needed);
     if (res == ERROR_INVALID_PARAMETER) {
-        pXcvClosePort(hXcv);
         skip("'MonitorUI' nor supported\n");
         return;
     }
@@ -642,7 +689,6 @@ static void test_XcvDataPort_MonitorUI(void)
         " and '<= MAX_PATH')\n", res, GetLastError(), needed);
 
     if (needed > MAX_PATH) {
-        pXcvClosePort(hXcv);
         skip("buffer overflow (%u)\n", needed);
         return;
     }
@@ -704,26 +750,16 @@ static void test_XcvDataPort_MonitorUI(void)
         "(expected 'ERROR_SUCCESS')\n", res, GetLastError(), needed);
 
 
-    pXcvClosePort(hXcv);
-
-
     /* small check without access-rights: */
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, 0, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
+    if (!hXcv_noaccess) return;
 
     /* The ACCESS_MASK is ignored for "MonitorUI" */
     memset(buffer, 0, len);
     needed = (DWORD) 0xdeadbeef;
     SetLastError(0xdeadbeef);
-    res = pXcvDataPort(hXcv, cmd_MonitorUIW, NULL, 0, buffer, sizeof(buffer), &needed);
+    res = pXcvDataPort(hXcv_noaccess, cmd_MonitorUIW, NULL, 0, buffer, sizeof(buffer), &needed);
     ok( res == ERROR_SUCCESS, "returned %d with %u and 0x%x "
         "(expected 'ERROR_SUCCESS')\n", res, GetLastError(), needed);
-
-    pXcvClosePort(hXcv);
-
 }
 
 /* ########################### */
@@ -731,24 +767,13 @@ static void test_XcvDataPort_MonitorUI(void)
 static void test_XcvDataPort_PortIsValid(void)
 {
     DWORD   res;
-    HANDLE  hXcv;
     DWORD   needed;
 
-
-    if ((pXcvOpenPort == NULL) || (pXcvDataPort == NULL) || (pXcvClosePort == NULL)) return;
-
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "hXcv: %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
-
     /* normal use: "LPT1:" */
     needed = (DWORD) 0xdeadbeef;
     SetLastError(0xdeadbeef);
     res = pXcvDataPort(hXcv, cmd_PortIsValidW, (PBYTE) portname_lpt1W, sizeof(portname_lpt1W), NULL, 0, &needed);
     if (res == ERROR_INVALID_PARAMETER) {
-        pXcvClosePort(hXcv);
         skip("'PostIsValid' not supported\n");
         return;
     }
@@ -865,24 +890,16 @@ static void test_XcvDataPort_PortIsValid(void)
         "returned %d with %u and 0x%x (expected ERROR_SUCCESS)\n",
         res, GetLastError(), needed);
 
-    pXcvClosePort(hXcv);
-
 
     /* small check without access-rights: */
-    hXcv = (HANDLE) 0xdeadbeef;
-    SetLastError(0xdeadbeef);
-    res = pXcvOpenPort(emptyW, 0, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (!res) return;
+    if (!hXcv_noaccess) return;
 
     /* The ACCESS_MASK from XcvOpenPort is ignored in "PortIsValid" */
     needed = (DWORD) 0xdeadbeef;
     SetLastError(0xdeadbeef);
-    res = pXcvDataPort(hXcv, cmd_PortIsValidW, (PBYTE) portname_lpt1W, sizeof(portname_lpt1W), NULL, 0, &needed);
+    res = pXcvDataPort(hXcv_noaccess, cmd_PortIsValidW, (PBYTE) portname_lpt1W, sizeof(portname_lpt1W), NULL, 0, &needed);
     ok( res == ERROR_SUCCESS, "returned %d with %u (expected ERROR_SUCCESS)\n", res, GetLastError());
 
-    pXcvClosePort(hXcv);
-
 }
 
 /* ########################### */
@@ -890,46 +907,45 @@ static void test_XcvDataPort_PortIsValid(void)
 static void test_XcvOpenPort(void)
 {
     DWORD   res;
-    HANDLE  hXcv;
+    HANDLE  hXcv2;
 
-    if ((pXcvOpenPort == NULL) || (pXcvClosePort == NULL)) return;
 
     if (0)
     {
     /* crash with native localspl.dll (w2k+xp) */
-    res = pXcvOpenPort(NULL, SERVER_ACCESS_ADMINISTER, &hXcv);
+    res = pXcvOpenPort(NULL, SERVER_ACCESS_ADMINISTER, &hXcv2);
     res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, NULL);
     }
 
 
     /* The returned handle is the result from a previous "spoolss.dll,DllAllocSplMem" */
     SetLastError(0xdeadbeef);
-    hXcv = (HANDLE) 0xdeadbeef;
-    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (res) pXcvClosePort(hXcv);
+    hXcv2 = (HANDLE) 0xdeadbeef;
+    res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv2);
+    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv2);
+    if (res) pXcvClosePort(hXcv2);
 
 
     /* The ACCESS_MASK is not checked in XcvOpenPort */
     SetLastError(0xdeadbeef);
-    hXcv = (HANDLE) 0xdeadbeef;
-    res = pXcvOpenPort(emptyW, 0, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (res) pXcvClosePort(hXcv);
+    hXcv2 = (HANDLE) 0xdeadbeef;
+    res = pXcvOpenPort(emptyW, 0, &hXcv2);
+    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv2);
+    if (res) pXcvClosePort(hXcv2);
 
 
     /* A copy of pszObject is saved in the Memory-Block */
     SetLastError(0xdeadbeef);
-    hXcv = (HANDLE) 0xdeadbeef;
-    res = pXcvOpenPort(portname_lpt1W, SERVER_ALL_ACCESS, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (res) pXcvClosePort(hXcv);
+    hXcv2 = (HANDLE) 0xdeadbeef;
+    res = pXcvOpenPort(portname_lpt1W, SERVER_ALL_ACCESS, &hXcv2);
+    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv2);
+    if (res) pXcvClosePort(hXcv2);
 
     SetLastError(0xdeadbeef);
-    hXcv = (HANDLE) 0xdeadbeef;
-    res = pXcvOpenPort(portname_fileW, SERVER_ALL_ACCESS, &hXcv);
-    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
-    if (res) pXcvClosePort(hXcv);
+    hXcv2 = (HANDLE) 0xdeadbeef;
+    res = pXcvOpenPort(portname_fileW, SERVER_ALL_ACCESS, &hXcv2);
+    ok(res, "returned %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv2);
+    if (res) pXcvClosePort(hXcv2);
 
 }
 
@@ -998,22 +1014,26 @@ START_TEST(localmon)
         GET_MONITOR_FUNC(XcvOpenPort);
         GET_MONITOR_FUNC(XcvDataPort);
         GET_MONITOR_FUNC(XcvClosePort);
+
+        if ((pXcvOpenPort) && (pXcvDataPort) && (pXcvClosePort)) {
+            SetLastError(0xdeadbeef);
+            res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
+            ok(res, "hXcv: %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv);
+
+            SetLastError(0xdeadbeef);
+            res = pXcvOpenPort(emptyW, 0, &hXcv_noaccess);
+            ok(res, "hXcv_noaccess: %d with %u and %p (expected '!= 0')\n", res, GetLastError(), hXcv_noaccess);
+        }
     }
 
-    if ((!pInitializePrintMonitorUI) && (pXcvOpenPort) && (pXcvDataPort) && (pXcvClosePort)) {
+    if ((!pInitializePrintMonitorUI) && (hXcv)) {
         /* The user interface for "Local Ports" is in a separate dll since w2k */
         BYTE    buffer[MAX_PATH];
-        DWORD   res;
         DWORD   len;
-        HANDLE  hXcv;
 
-        res = pXcvOpenPort(emptyW, SERVER_ACCESS_ADMINISTER, &hXcv);
-        if (res) {
-            res = pXcvDataPort(hXcv, cmd_MonitorUIW, NULL, 0, buffer, MAX_PATH, &len);
-            if (res == ERROR_SUCCESS) hlocalui = LoadLibraryW( (LPWSTR) buffer);
-            if (hlocalui) pInitializePrintMonitorUI = (void *) GetProcAddress(hlocalui, "InitializePrintMonitorUI");
-            pXcvClosePort(hXcv);
-        }
+        res = pXcvDataPort(hXcv, cmd_MonitorUIW, NULL, 0, buffer, MAX_PATH, &len);
+        if (res == ERROR_SUCCESS) hlocalui = LoadLibraryW( (LPWSTR) buffer);
+        if (hlocalui) pInitializePrintMonitorUI = (void *) GetProcAddress(hlocalui, "InitializePrintMonitorUI");
     }
 
     if (pInitializePrintMonitorUI) {
@@ -1037,10 +1057,24 @@ START_TEST(localmon)
     test_ConfigurePort();
     test_DeletePort();
     test_EnumPorts();
-    test_XcvClosePort();
-    test_XcvDataPort_ConfigureLPTPortCommandOK();
-    test_XcvDataPort_GetTransmissionRetryTimeout();
-    test_XcvDataPort_MonitorUI();
-    test_XcvDataPort_PortIsValid();
-    test_XcvOpenPort();
+    if ( !hXcv ) {
+        skip("Xcv not supported\n");
+    }
+    else
+    {
+        test_XcvClosePort();
+        test_XcvDataPort_AddPort();
+        test_XcvDataPort_ConfigureLPTPortCommandOK();
+        test_XcvDataPort_DeletePort();
+        test_XcvDataPort_GetTransmissionRetryTimeout();
+        test_XcvDataPort_MonitorUI();
+        test_XcvDataPort_PortIsValid();
+        test_XcvOpenPort();
+
+        pXcvClosePort(hXcv);
+    }
+    if (hXcv_noaccess) pXcvClosePort(hXcv_noaccess);
+
+    /* Cleanup our temporary file */
+    DeleteFileW(tempfileW);
 }
