@@ -446,7 +446,7 @@ static VOID set_file_source(MSIPACKAGE* package, MSIFILE* file, LPCWSTR path)
     if (!file->IsCompressed)
     {
         LPWSTR p, path;
-        p = resolve_folder(package, file->Component->Directory, TRUE, FALSE, NULL);
+        p = resolve_folder(package, file->Component->Directory, TRUE, FALSE, TRUE, NULL);
         path = build_directory_name(2, p, file->ShortName);
         if (INVALID_FILE_ATTRIBUTES == GetFileAttributesW( path ))
         {
@@ -638,13 +638,9 @@ static void schedule_install_files(MSIPACKAGE *package)
     }
 }
 
-static UINT copy_install_file(MSIFILE *file)
+static UINT copy_file(MSIFILE *file)
 {
     BOOL ret;
-    UINT gle;
-
-    TRACE("Copying %s to %s\n", debugstr_w(file->SourcePath),
-          debugstr_w(file->TargetPath));
 
     ret = CopyFileW(file->SourcePath, file->TargetPath, FALSE);
     if (ret)
@@ -653,7 +649,20 @@ static UINT copy_install_file(MSIFILE *file)
         return ERROR_SUCCESS;
     }
 
-    gle = GetLastError();
+    return GetLastError();
+}
+
+static UINT copy_install_file(MSIFILE *file)
+{
+    UINT gle;
+
+    TRACE("Copying %s to %s\n", debugstr_w(file->SourcePath),
+          debugstr_w(file->TargetPath));
+
+    gle = copy_file(file);
+    if (gle == ERROR_SUCCESS)
+        return gle;
+
     if (gle == ERROR_ALREADY_EXISTS && file->state == msifs_overwrite)
     {
         TRACE("overwriting existing file\n");
@@ -665,6 +674,13 @@ static UINT copy_install_file(MSIFILE *file)
         TRACE("Source file not found\n");
         gle = ERROR_SUCCESS;
     }
+    else if (gle == ERROR_ACCESS_DENIED)
+    {
+        SetFileAttributesW(file->TargetPath, FILE_ATTRIBUTE_NORMAL);
+
+        gle = copy_file(file);
+        TRACE("Overwriting existing file: %d\n", gle);
+    }
     else if (!(file->Attributes & msidbFileAttributesVital))
     {
         TRACE("Ignoring error for nonvital\n");
@@ -829,7 +845,7 @@ static UINT ITERATE_DuplicateFiles(MSIRECORD *row, LPVOID param)
     {
         LPCWSTR destkey;
         destkey = MSI_RecordGetString(row,5);
-        dest_path = resolve_folder(package, destkey, FALSE,FALSE,NULL);
+        dest_path = resolve_folder(package, destkey, FALSE, FALSE, TRUE, NULL);
         if (!dest_path)
         {
             /* try a Property */
