@@ -36,7 +36,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(ntdll);
 void dump_ObjectAttributes (const OBJECT_ATTRIBUTES *oa)
 {
 	if (oa)
-	  TRACE("%p:(name=%s, attr=0x%08lx, hRoot=%p, sd=%p)\n",
+	  TRACE("%p:(name=%s, attr=0x%08x, hRoot=%p, sd=%p)\n",
 	    oa, debugstr_us(oa->ObjectName),
 	    oa->Attributes, oa->RootDirectory, oa->SecurityDescriptor);
 }
@@ -149,7 +149,7 @@ int CDECL NTDLL_abs( int i )
 /*********************************************************************
  *                  labs   (NTDLL.@)
  */
-long int CDECL NTDLL_labs( long int i )
+long CDECL NTDLL_labs( long i )
 {
     return labs( i );
 }
