@@ -652,23 +652,175 @@ static int wait_reply( void *cookie )
 }
 
 
+/***********************************************************************
+ *              invoke_apc
+ *
+ * Invoke a single APC. Return TRUE if a user APC has been run.
+ */
+static BOOL invoke_apc( const apc_call_t *call, apc_result_t *result )
+{
+    BOOL user_apc = FALSE;
+
+    memset( result, 0, sizeof(*result) );
+
+    switch (call->type)
+    {
+    case APC_USER:
+        call->user.func( call->user.args[0], call->user.args[1], call->user.args[2] );
+        user_apc = TRUE;
+        break;
+    case APC_TIMER:
+    {
+        LARGE_INTEGER time;
+        /* convert sec/usec to NT time */
+        RtlSecondsSince1970ToTime( call->timer.time.sec, &time );
+        time.QuadPart += call->timer.time.usec * 10;
+        call->timer.func( call->timer.arg, time.u.LowPart, time.u.HighPart );
+        user_apc = TRUE;
+        break;
+    }
+    case APC_ASYNC_IO:
+        NtCurrentTeb()->num_async_io--;
+        call->async_io.func( call->async_io.user, call->async_io.sb, call->async_io.status );
+        result->type = call->type;
+        result->async_io.status = ((IO_STATUS_BLOCK *)call->async_io.sb)->u.Status;
+        break;
+    case APC_VIRTUAL_ALLOC:
+        result->type = call->type;
+        result->virtual_alloc.addr = call->virtual_alloc.addr;
+        result->virtual_alloc.size = call->virtual_alloc.size;
+        result->virtual_alloc.status = NtAllocateVirtualMemory( NtCurrentProcess(),
+                                                                &result->virtual_alloc.addr,
+                                                                call->virtual_alloc.zero_bits,
+                                                                &result->virtual_alloc.size,
+                                                                call->virtual_alloc.op_type,
+                                                                call->virtual_alloc.prot );
+        break;
+    case APC_VIRTUAL_FREE:
+        result->type = call->type;
+        result->virtual_free.addr = call->virtual_free.addr;
+        result->virtual_free.size = call->virtual_free.size;
+        result->virtual_free.status = NtFreeVirtualMemory( NtCurrentProcess(),
+                                                           &result->virtual_free.addr,
+                                                           &result->virtual_free.size,
+                                                           call->virtual_free.op_type );
+        break;
+    case APC_VIRTUAL_QUERY:
+    {
+        MEMORY_BASIC_INFORMATION info;
+        result->type = call->type;
+        result->virtual_query.status = NtQueryVirtualMemory( NtCurrentProcess(),
+                                                             call->virtual_query.addr,
+                                                             MemoryBasicInformation, &info,
+                                                             sizeof(info), NULL );
+        if (result->virtual_query.status == STATUS_SUCCESS)
+        {
+            result->virtual_query.base       = info.BaseAddress;
+            result->virtual_query.alloc_base = info.AllocationBase;
+            result->virtual_query.size       = info.RegionSize;
+            result->virtual_query.state      = info.State;
+            result->virtual_query.prot       = info.Protect;
+            result->virtual_query.alloc_prot = info.AllocationProtect;
+            result->virtual_query.alloc_type = info.Type;
+        }
+        break;
+    }
+    case APC_VIRTUAL_PROTECT:
+        result->type = call->type;
+        result->virtual_protect.addr = call->virtual_protect.addr;
+        result->virtual_protect.size = call->virtual_protect.size;
+        result->virtual_protect.status = NtProtectVirtualMemory( NtCurrentProcess(),
+                                                                 &result->virtual_protect.addr,
+                                                                 &result->virtual_protect.size,
+                                                                 call->virtual_protect.prot,
+                                                                 &result->virtual_protect.prot );
+        break;
+    case APC_VIRTUAL_FLUSH:
+        result->type = call->type;
+        result->virtual_flush.addr = call->virtual_flush.addr;
+        result->virtual_flush.size = call->virtual_flush.size;
+        result->virtual_flush.status = NtFlushVirtualMemory( NtCurrentProcess(),
+                                                             &result->virtual_flush.addr,
+                                                             &result->virtual_flush.size, 0 );
+        break;
+    case APC_VIRTUAL_LOCK:
+        result->type = call->type;
+        result->virtual_lock.addr = call->virtual_lock.addr;
+        result->virtual_lock.size = call->virtual_lock.size;
+        result->virtual_lock.status = NtLockVirtualMemory( NtCurrentProcess(),
+                                                           &result->virtual_lock.addr,
+                                                           &result->virtual_lock.size, 0 );
+        break;
+    case APC_VIRTUAL_UNLOCK:
+        result->type = call->type;
+        result->virtual_unlock.addr = call->virtual_unlock.addr;
+        result->virtual_unlock.size = call->virtual_unlock.size;
+        result->virtual_unlock.status = NtUnlockVirtualMemory( NtCurrentProcess(),
+                                                               &result->virtual_unlock.addr,
+                                                               &result->virtual_unlock.size, 0 );
+        break;
+    case APC_MAP_VIEW:
+    {
+        LARGE_INTEGER offset;
+        result->type = call->type;
+        result->map_view.addr   = call->map_view.addr;
+        result->map_view.size   = call->map_view.size;
+        offset.u.LowPart        = call->map_view.offset_low;
+        offset.u.HighPart       = call->map_view.offset_high;
+        result->map_view.status = NtMapViewOfSection( call->map_view.handle, NtCurrentProcess(),
+                                                      &result->map_view.addr, call->map_view.zero_bits,
+                                                      0, &offset, &result->map_view.size, ViewShare,
+                                                      call->map_view.alloc_type, call->map_view.prot );
+        NtClose( call->map_view.handle );
+        break;
+    }
+    case APC_UNMAP_VIEW:
+        result->type = call->type;
+        result->unmap_view.status = NtUnmapViewOfSection( NtCurrentProcess(), call->unmap_view.addr );
+        break;
+    case APC_CREATE_THREAD:
+    {
+        CLIENT_ID id;
+        result->type = call->type;
+        result->create_thread.status = RtlCreateUserThread( NtCurrentProcess(), NULL,
+                                                            call->create_thread.suspend, NULL,
+                                                            call->create_thread.reserve,
+                                                            call->create_thread.commit,
+                                                            call->create_thread.func,
+                                                            call->create_thread.arg,
+                                                            &result->create_thread.handle, &id );
+        result->create_thread.tid = (thread_id_t)id.UniqueThread;
+        break;
+    }
+    default:
+        server_protocol_error( "get_apc_request: bad type %d\n", call->type );
+        break;
+    }
+    return user_apc;
+}
+
 /***********************************************************************
  *              call_apcs
  *
- * Call outstanding APCs.
+ * Call outstanding APCs. Return TRUE if a user APC has been run.
  */
-static void call_apcs( BOOL alertable )
+static BOOL call_apcs( BOOL alertable )
 {
+    BOOL user_apc = FALSE;
     NTSTATUS ret;
     apc_call_t call;
+    apc_result_t result;
     HANDLE handle = 0;
 
+    memset( &result, 0, sizeof(result) );
+
     for (;;)
     {
         SERVER_START_REQ( get_apc )
         {
             req->alertable = alertable;
             req->prev      = handle;
+            req->result    = result;
             if (!(ret = wine_server_call( req )))
             {
                 handle = reply->handle;
@@ -677,30 +829,56 @@ static void call_apcs( BOOL alertable )
         }
         SERVER_END_REQ;
 
-        if (ret) return;  /* no more APCs */
+        if (ret) return user_apc;  /* no more APCs */
 
-        switch (call.type)
+        user_apc = invoke_apc( &call, &result );
+    }
+}
+
+
+/***********************************************************************
+ *           NTDLL_queue_process_apc
+ */
+NTSTATUS NTDLL_queue_process_apc( HANDLE process, const apc_call_t *call, apc_result_t *result )
+{
+    for (;;)
+    {
+        NTSTATUS ret;
+        HANDLE handle = 0;
+        BOOL self = FALSE;
+
+        SERVER_START_REQ( queue_apc )
         {
-        case APC_USER:
-            call.user.func( call.user.args[0], call.user.args[1], call.user.args[2] );
-            break;
-        case APC_TIMER:
+            req->process = process;
+            req->call = *call;
+            if (!(ret = wine_server_call( req )))
+            {
+                handle = reply->handle;
+                self = reply->self;
+            }
+        }
+        SERVER_END_REQ;
+        if (ret != STATUS_SUCCESS) return ret;
+
+        if (self)
         {
-            LARGE_INTEGER time;
-            /* convert sec/usec to NT time */
-            RtlSecondsSince1970ToTime( call.timer.time.sec, &time );
-            time.QuadPart += call.timer.time.usec * 10;
-            call.timer.func( call.timer.arg, time.u.LowPart, time.u.HighPart );
-            break;
+            invoke_apc( call, result );
         }
-        case APC_ASYNC_IO:
-            NtCurrentTeb()->num_async_io--;
-            call.async_io.func( call.async_io.user, call.async_io.sb, call.async_io.status );
-            break;
-        default:
-            server_protocol_error( "get_apc_request: bad type %d\n", call.type );
-            break;
+        else
+        {
+            NtWaitForSingleObject( handle, FALSE, NULL );
+
+            SERVER_START_REQ( get_apc_result )
+            {
+                req->handle = handle;
+                if (!(ret = wine_server_call( req ))) *result = reply->result;
+            }
+            SERVER_END_REQ;
+
+            if (!ret && result->type == APC_NONE) continue;  /* APC didn't run, try again */
+            if (ret) NtClose( handle );
         }
+        return ret;
     }
 }
 
@@ -715,7 +893,9 @@ NTSTATUS NTDLL_wait_for_multiple_objects( UINT count, const HANDLE *handles, UIN
 {
     NTSTATUS ret;
     int cookie;
+    abs_time_t abs_timeout;
 
+    NTDLL_get_server_abstime( &abs_timeout, timeout );
     if (timeout) flags |= SELECT_TIMEOUT;
     for (;;)
     {
@@ -724,15 +904,14 @@ NTSTATUS NTDLL_wait_for_multiple_objects( UINT count, const HANDLE *handles, UIN
             req->flags   = flags;
             req->cookie  = &cookie;
             req->signal  = signal_object;
-            NTDLL_get_server_abstime( &req->timeout, timeout );
+            req->timeout = abs_timeout;
             wine_server_add_data( req, handles, count * sizeof(HANDLE) );
             ret = wine_server_call( req );
         }
         SERVER_END_REQ;
         if (ret == STATUS_PENDING) ret = wait_reply( &cookie );
         if (ret != STATUS_USER_APC) break;
-        call_apcs( (flags & SELECT_ALERTABLE) != 0 );
-        if (flags & SELECT_ALERTABLE) break;
+        if (call_apcs( (flags & SELECT_ALERTABLE) != 0 )) break;
         signal_object = 0;  /* don't signal it multiple times */
     }
 
