@@ -275,6 +275,12 @@ static const struct tagTZ_INFO TZ_INFO[] =
    {"YEKST",
     {'U','r','a','l','s',' ','D','a','y','l','i','g','h','t',
      ' ','T','i','m','e',' ','(','S','u','m','m','e','r',')','\0'}, -360, 1},
+   {"OMST",
+    {'O','m','s','k',' ','S','t','a','n','d','a','r','d',
+     ' ','T','i','m','e',' ','(','W','i','n','t','e','r',')','\0'}, -360, 0},
+   {"OMSST",
+    {'O','m','s','k',' ','D','a','y','l','i','g','h','t',
+     ' ','T','i','m','e',' ','(','S','u','m','m','e','r',')','\0'}, -420, 1},
    {"PKT",
     {'W','e','s','t',' ','A','s','i','a',' ','S','t','a','n','d','a','r','d',
      ' ','T','i','m','e','\0'}, -300, 0},
@@ -320,6 +326,9 @@ static const struct tagTZ_INFO TZ_INFO[] =
    {"WST",
     {'W','.',' ','A','u','s','t','r','a','l','i','a',' ','S','t','a','n','d',
      'a','r','d',' ','T','i','m','e','\0'}, -480, 0},
+   {"WST",
+    {'W','.',' ','A','u','s','t','r','a','l','i','a',' ','S','u','m','m','e',
+     'r',' ','T','i','m','e','\0'}, -540, 1},
    {"JST",
     {'T','o','k','y','o',' ','S','t','a','n','d','a','r','d',' ','T','i','m',
      'e','\0'}, -540, 0},
@@ -385,6 +394,9 @@ static const struct tagTZ_INFO TZ_INFO[] =
      ' ','T','i','m','e','\0'}, -480, 0},
    {"UYT",
     {'U','r','u','g','u','a','y','a','n',' ','T','i','m','e','\0'}, 180, 0},
+   {"UYST",
+    {'U','r','u','g','u','a','y','a','n',' ','S','u','m','m','e','r',' ','T',
+    'i','m','e','\0'}, 120, 1},
    {"MYT",
     {'M','a','l','a','y','s','i','a','n',' ','T','i','m','e','\0'}, -480, 0},
    {"PHT",
@@ -435,11 +447,11 @@ static inline int IsLeapYear(int Year)
 }
 
 /***********************************************************************
- *              NTDLL_get_server_timeout
+ *              NTDLL_get_server_abstime
  *
- * Convert a NTDLL timeout into a timeval struct to send to the server.
+ * Convert a NTDLL time into an abs_time_t struct to send to the server.
  */
-void NTDLL_get_server_timeout( abs_time_t *when, const LARGE_INTEGER *timeout )
+void NTDLL_get_server_abstime( abs_time_t *when, const LARGE_INTEGER *timeout )
 {
     UINT remainder;
 
@@ -484,14 +496,14 @@ void NTDLL_get_server_timeout( abs_time_t *when, const LARGE_INTEGER *timeout )
 
 
 /***********************************************************************
- *              NTDLL_from_server_timeout
+ *              NTDLL_from_server_abstime
  *
- * Convert a timeval struct from the server into an NTDLL timeout.
+ * Convert a timeval struct from the server into an NTDLL time.
  */
-void NTDLL_from_server_timeout( LARGE_INTEGER *timeout, const abs_time_t *when )
+void NTDLL_from_server_abstime( LARGE_INTEGER *time, const abs_time_t *when )
 {
-    timeout->QuadPart = when->sec * (ULONGLONG)TICKSPERSEC + TICKS_1601_TO_1970;
-    timeout->QuadPart += when->usec * 10;
+    time->QuadPart = when->sec * (ULONGLONG)TICKSPERSEC + TICKS_1601_TO_1970;
+    time->QuadPart += when->usec * 10;
 }
 
 
@@ -874,7 +886,8 @@ NTSTATUS WINAPI NtQueryPerformanceCounter( PLARGE_INTEGER Counter, PLARGE_INTEGE
      * to one of 1.193182 MHz, with some care for arithmetic
      * overflow ( will not overflow for 5000 years ) and
      * good accuracy ( 105/88 = 1.19318182) */
-    Counter->QuadPart = (((now.tv_sec - server_start_time) * (ULONGLONG)1000000 + now.tv_usec) * 105) / 88;
+    Counter->QuadPart = (((now.tv_sec - server_start_time.sec) * (ULONGLONG)1000000 +
+                          (now.tv_usec - server_start_time.usec)) * 105) / 88;
     if (Frequency) Frequency->QuadPart = 1193182;
     return STATUS_SUCCESS;
 }
@@ -889,7 +902,8 @@ ULONG WINAPI NtGetTickCount(void)
     struct timeval current_time;
 
     gettimeofday(&current_time, NULL);
-    return (current_time.tv_sec - server_start_time)*1000 + current_time.tv_usec/1000;
+    return (current_time.tv_sec - server_start_time.sec) * 1000 +
+           (current_time.tv_usec - server_start_time.usec) / 1000;
 }
 
 
