@@ -933,6 +933,34 @@ HRESULT WINAPI OleSetContainedObject(
   return S_OK;
 }
 
+/******************************************************************************
+ *              OleRun        [OLE32.@]
+ *
+ * Set the OLE object to the running state.
+ *
+ * PARAMS
+ *  pUnknown [I] OLE object to run.
+ *
+ * RETURNS
+ *  Success: S_OK.
+ *  Failure: Any HRESULT code.
+ */
+HRESULT WINAPI OleRun(LPUNKNOWN pUnknown)
+{
+    IRunnableObject *runable;
+    HRESULT hres;
+
+    TRACE("(%p)\n", pUnknown);
+
+    hres = IUnknown_QueryInterface(pUnknown, &IID_IRunnableObject, (void**)&runable);
+    if (FAILED(hres))
+        return S_OK; /* Appears to return no error. */
+
+    hres = IRunnableObject_Run(runable, NULL);
+    IRunnableObject_Release(runable);
+    return hres;
+}
+
 /******************************************************************************
  *              OleLoad        [OLE32.@]
  */
@@ -948,7 +976,7 @@ HRESULT WINAPI OleLoad(
   STATSTG          storageInfo;
   HRESULT          hres;
 
-  TRACE("(%p,%p,%p,%p)\n", pStg, riid, pClientSite, ppvObj);
+  TRACE("(%p, %s, %p, %p)\n", pStg, debugstr_guid(riid), pClientSite, ppvObj);
 
   *ppvObj = NULL;
 
@@ -966,7 +994,7 @@ HRESULT WINAPI OleLoad(
    */
   hres = CoCreateInstance(&storageInfo.clsid,
 			  NULL,
-			  CLSCTX_INPROC_HANDLER,
+			  CLSCTX_INPROC_HANDLER|CLSCTX_INPROC_SERVER,
 			  riid,
 			  (void**)&pUnk);
 
@@ -2325,9 +2353,10 @@ HRESULT WINAPI OleCreate(
     IUnknown * pUnk = NULL;
     IOleObject *pOleObject = NULL;
 
-    FIXME("\n\t%s\n\t%s semi-stub!\n", debugstr_guid(rclsid), debugstr_guid(riid));
+    TRACE("(%s, %s, %d, %p, %p, %p, %p)\n", debugstr_guid(rclsid),
+        debugstr_guid(riid), renderopt, pFormatEtc, pClientSite, pStg, ppvObj);
 
-    hres = CoCreateInstance(rclsid, 0, CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER|CLSCTX_LOCAL_SERVER , riid, (LPVOID*)&pUnk);
+    hres = CoCreateInstance(rclsid, 0, CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER, riid, (LPVOID*)&pUnk);
 
     if (SUCCEEDED(hres))
         hres = IStorage_SetClass(pStg, rclsid);
