@@ -23,6 +23,7 @@
 #define CONST_VTABLE
 
 #include <stdarg.h>
+#include <stdio.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -47,9 +48,25 @@ static const IID IID_IWineTest =
     {0xa1, 0xa2, 0x5d, 0x5a, 0x36, 0x54, 0xd3, 0xbd}
 }; /* 5201163f-8164-4fd0-a1a2-5d5a3654d3bd */
 
+static const IID IID_IRemUnknown =
+{
+    0x00000131,
+    0x0000,
+    0x0000,
+    {0xc0,0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46}
+};
+
 #define EXTENTID_WineTest IID_IWineTest
 #define CLSID_WineTest IID_IWineTest
 
+static const CLSID CLSID_WineOOPTest =
+{
+    0x5201163f,
+    0x8164,
+    0x4fd0,
+    {0xa1, 0xa2, 0x5d, 0x5a, 0x36, 0x54, 0xd3, 0xbd}
+}; /* 5201163f-8164-4fd0-a1a2-5d5a3654d3bd */
+
 static void test_cocreateinstance_proxy(void)
 {
     IUnknown *pProxy;
@@ -132,7 +149,9 @@ static HRESULT WINAPI Test_IClassFactory_QueryInterface(
     if (ppvObj == NULL) return E_POINTER;
 
     if (IsEqualGUID(riid, &IID_IUnknown) ||
-        IsEqualGUID(riid, &IID_IClassFactory))
+        IsEqualGUID(riid, &IID_IClassFactory) ||
+        /* the only other interface Wine is currently able to marshal (for testing two proxies) */
+        IsEqualGUID(riid, &IID_IRemUnknown))
     {
         *ppvObj = (LPVOID)iface;
         IClassFactory_AddRef(iface);
@@ -1396,7 +1415,7 @@ static void test_proxy_interfaces(void)
     if (hr == S_OK) IUnknown_Release(pOtherUnknown);
 
     hr = IUnknown_QueryInterface(pProxy, &IID_IClientSecurity, (LPVOID*)&pOtherUnknown);
-    todo_wine { ok_ole_success(hr, IUnknown_QueryInterface IID_IClientSecurity); }
+    ok_ole_success(hr, IUnknown_QueryInterface IID_IClientSecurity);
     if (hr == S_OK) IUnknown_Release(pOtherUnknown);
 
     hr = IUnknown_QueryInterface(pProxy, &IID_IMultiQI, (LPVOID*)&pOtherUnknown);
@@ -1438,7 +1457,6 @@ static HRESULT WINAPI HeapUnknown_QueryInterface(IUnknown *iface, REFIID riid, v
 static ULONG WINAPI HeapUnknown_AddRef(IUnknown *iface)
 {
     HeapUnknown *This = (HeapUnknown *)iface;
-    trace("HeapUnknown_AddRef(%p)\n", iface);
     return InterlockedIncrement((LONG*)&This->refs);
 }
 
@@ -1446,7 +1464,6 @@ static ULONG WINAPI HeapUnknown_Release(IUnknown *iface)
 {
     HeapUnknown *This = (HeapUnknown *)iface;
     ULONG refs = InterlockedDecrement((LONG*)&This->refs);
-    trace("HeapUnknown_Release(%p)\n", iface);
     if (!refs) HeapFree(GetProcessHeap(), 0, This);
     return refs;
 }
@@ -2103,6 +2120,99 @@ static void test_handler_marshaling(void)
 }
 
 
+static void test_client_security(void)
+{
+    HRESULT hr;
+    IStream *pStream = NULL;
+    IClassFactory *pProxy = NULL;
+    IUnknown *pProxy2 = NULL;
+    IUnknown *pUnknown1 = NULL;
+    IUnknown *pUnknown2 = NULL;
+    IClientSecurity *pCliSec = NULL;
+    IMarshal *pMarshal;
+    DWORD tid;
+    HANDLE thread;
+    static const LARGE_INTEGER ullZero;
+    DWORD dwAuthnSvc;
+    DWORD dwAuthzSvc;
+    OLECHAR *pServerPrincName;
+    DWORD dwAuthnLevel;
+    DWORD dwImpLevel;
+    void *pAuthInfo;
+    DWORD dwCapabilities;
+    void *pv;
+
+    cLocks = 0;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &pStream);
+    ok_ole_success(hr, "CreateStreamOnHGlobal");
+    tid = start_host_object(pStream, &IID_IClassFactory, (IUnknown*)&Test_ClassFactory, MSHLFLAGS_NORMAL, &thread);
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy);
+    ok_ole_success(hr, "CoUnmarshalInterface");
+    IStream_Release(pStream);
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IUnknown, (LPVOID*)&pUnknown1);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IUnknown");
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IRemUnknown, (LPVOID*)&pProxy2);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IStream");
+
+    hr = IUnknown_QueryInterface(pProxy2, &IID_IUnknown, (LPVOID*)&pUnknown2);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IUnknown");
+
+    ok(pUnknown1 == pUnknown2, "both proxy's IUnknowns should be the same - %p, %p\n", pUnknown1, pUnknown2);
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IMarshal, (LPVOID*)&pMarshal);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IMarshal");
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IClientSecurity, (LPVOID*)&pCliSec);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IClientSecurity");
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pProxy, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket (all NULLs)");
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pMarshal, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    todo_wine ok(hr == E_NOINTERFACE, "IClientSecurity_QueryBlanket with local interface should have returned E_NOINTERFACE instead of 0x%08x\n", hr);
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pProxy, &dwAuthnSvc, &dwAuthzSvc, &pServerPrincName, &dwAuthnLevel, &dwImpLevel, &pAuthInfo, &dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket");
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pProxy, dwAuthnSvc, dwAuthzSvc, pServerPrincName, dwAuthnLevel, RPC_C_IMP_LEVEL_IMPERSONATE, pAuthInfo, dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_SetBlanket");
+
+    hr = IClassFactory_CreateInstance(pProxy, NULL, &IID_IWineTest, &pv);
+    ok(hr == E_NOINTERFACE, "COM call should have succeeded instead of returning 0x%08x\n", hr);
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pMarshal, dwAuthnSvc, dwAuthzSvc, pServerPrincName, dwAuthnLevel, dwImpLevel, pAuthInfo, dwCapabilities);
+    todo_wine ok(hr == E_NOINTERFACE, "IClientSecurity_SetBlanket with local interface should have returned E_NOINTERFACE instead of 0x%08x\n", hr);
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pProxy, 0xdeadbeef, dwAuthzSvc, pServerPrincName, dwAuthnLevel, dwImpLevel, pAuthInfo, dwCapabilities);
+    todo_wine ok(hr == E_INVALIDARG, "IClientSecurity_SetBlanke with invalid dwAuthnSvc should have returned E_INVALIDARG instead of 0x%08x\n", hr);
+
+    CoTaskMemFree(pServerPrincName);
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pUnknown1, &dwAuthnSvc, &dwAuthzSvc, &pServerPrincName, &dwAuthnLevel, &dwImpLevel, &pAuthInfo, &dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket(IUnknown)");
+
+    CoTaskMemFree(pServerPrincName);
+
+    IClassFactory_Release(pProxy);
+#if 0
+    /* FIXME: fix stub manager to not depend on the IID of the interface to
+     * determine whether it is really the rem unknown for the object before
+     * re-enabling this line */
+    IUnknown_Release(pProxy2);
+#endif
+    IUnknown_Release(pUnknown1);
+    IUnknown_Release(pUnknown2);
+    IMarshal_Release(pMarshal);
+    IClientSecurity_Release(pCliSec);
+
+    end_host_object(tid, thread);
+}
+
 static HANDLE heventShutdown;
 
 static void LockModuleOOP(void)
@@ -2154,6 +2264,11 @@ static HRESULT WINAPI TestOOP_IClassFactory_CreateInstance(
     REFIID riid,
     LPVOID *ppvObj)
 {
+    if (IsEqualIID(riid, &IID_IClassFactory) || IsEqualIID(riid, &IID_IUnknown))
+    {
+        *ppvObj = iface;
+        return S_OK;
+    }
     return CLASS_E_CLASSNOTAVAILABLE;
 }
 
@@ -2179,19 +2294,72 @@ static const IClassFactoryVtbl TestClassFactoryOOP_Vtbl =
 
 static IClassFactory TestOOP_ClassFactory = { &TestClassFactoryOOP_Vtbl };
 
+static void test_register_local_server(void)
+{
+    DWORD cookie;
+    HRESULT hr;
+    HANDLE ready_event;
+    HANDLE quit_event;
+    DWORD wait;
+
+    heventShutdown = CreateEvent(NULL, TRUE, FALSE, NULL);
+
+    hr = CoRegisterClassObject(&CLSID_WineOOPTest, (IUnknown *)&TestOOP_ClassFactory,
+                               CLSCTX_LOCAL_SERVER, REGCLS_SINGLEUSE, &cookie);
+    ok_ole_success(hr, CoRegisterClassObject);
+
+    ready_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Ready Event");
+    SetEvent(ready_event);
+
+    quit_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Quit Event");
+
+    do
+    {
+        wait = MsgWaitForMultipleObjects(1, &quit_event, FALSE, INFINITE, QS_ALLINPUT);
+        if (wait == WAIT_OBJECT_0+1)
+        {
+            MSG msg;
+            BOOL ret = PeekMessage(&msg, NULL, 0, 0, PM_REMOVE);
+            if (ret)
+            {
+                trace("Message 0x%x\n", msg.message);
+                TranslateMessage(&msg);
+                DispatchMessage(&msg);
+            }
+        }
+    }
+    while (wait == WAIT_OBJECT_0+1);
+
+    hr = CoRevokeClassObject(cookie);
+    ok_ole_success(hr, CoRevokeClassObject);
+}
+
+static HANDLE create_target_process(const char *arg)
+{
+    char **argv;
+    char cmdline[MAX_PATH];
+    PROCESS_INFORMATION pi;
+    STARTUPINFO si = { 0 };
+    si.cb = sizeof(si);
+
+    winetest_get_mainargs( &argv );
+    sprintf(cmdline, "%s %s %s", argv[0], argv[1], arg);
+    ok(CreateProcess(argv[0], cmdline, NULL, NULL, FALSE, 0, NULL, NULL,
+                     &si, &pi) != 0, "error: %u\n", GetLastError());
+    ok(CloseHandle(pi.hThread) != 0, "error %u\n", GetLastError());
+    return pi.hProcess;
+}
+
 /* tests functions commonly used by out of process COM servers */
-static void test_out_of_process_com(void)
+static void test_local_server(void)
 {
-    static const CLSID CLSID_WineOOPTest = {
-        0x5201163f,
-        0x8164,
-        0x4fd0,
-        {0xa1, 0xa2, 0x5d, 0x5a, 0x36, 0x54, 0xd3, 0xbd}
-    }; /* 5201163f-8164-4fd0-a1a2-5d5a3654d3bd */
     DWORD cookie;
     HRESULT hr;
     IClassFactory * cf;
     DWORD ret;
+    HANDLE process;
+    HANDLE quit_event;
+    HANDLE ready_event;
 
     heventShutdown = CreateEvent(NULL, TRUE, FALSE, NULL);
 
@@ -2206,10 +2374,9 @@ static void test_out_of_process_com(void)
      * class in the registry */
     hr = CoGetClassObject(&CLSID_WineOOPTest, CLSCTX_INPROC_SERVER,
         NULL, &IID_IClassFactory, (LPVOID*)&cf);
-    todo_wine {
-    ok(hr == REGDB_E_CLASSNOTREG,
+    ok(hr == REGDB_E_CLASSNOTREG || /* NT */
+       hr == S_OK /* Win9x */,
         "CoGetClassObject should have returned REGDB_E_CLASSNOTREG instead of 0x%08x\n", hr);
-    }
 
     /* Resume the object suspended above ... */
     hr = CoResumeClassObjects();
@@ -2226,32 +2393,54 @@ static void test_out_of_process_com(void)
     ok_no_locks();
 
     hr = IClassFactory_LockServer(cf, TRUE);
-    trace("IClassFactory_LockServer returned 0x%08x\n", hr);
+    ok_ole_success(hr, IClassFactory_LockServer);
 
     ok_more_than_one_lock();
     
     IClassFactory_LockServer(cf, FALSE);
+    ok_ole_success(hr, IClassFactory_LockServer);
 
     ok_no_locks();
 
     IClassFactory_Release(cf);
 
     /* wait for shutdown signal */
-    ret = WaitForSingleObject(heventShutdown, 5000);
-    todo_wine { ok(ret != WAIT_TIMEOUT, "Server didn't shut down or machine is under very heavy load\n"); }
+    ret = WaitForSingleObject(heventShutdown, 0);
+    ok(ret != WAIT_TIMEOUT, "Server didn't shut down\n");
 
     /* try to connect again after SCM has suspended registered class objects */
     hr = CoGetClassObject(&CLSID_WineOOPTest, CLSCTX_INPROC_SERVER | CLSCTX_LOCAL_SERVER, NULL,
         &IID_IClassFactory, (LPVOID*)&cf);
-    todo_wine {
-    ok(hr == CO_E_SERVER_STOPPING,
+    ok(hr == CO_E_SERVER_STOPPING || /* NT */
+       hr == S_OK /* Win9x */,
         "CoGetClassObject should have returned CO_E_SERVER_STOPPING instead of 0x%08x\n", hr);
-    }
 
     hr = CoRevokeClassObject(cookie);
     ok_ole_success(hr, CoRevokeClassObject);
 
     CloseHandle(heventShutdown);
+
+    process = create_target_process("-Embedding");
+    ok(process != NULL, "couldn't start local server process, error was %d\n", GetLastError());
+
+    ready_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Ready Event");
+    WaitForSingleObject(ready_event, 1000);
+    CloseHandle(ready_event);
+
+    hr = CoCreateInstance(&CLSID_WineOOPTest, NULL, CLSCTX_LOCAL_SERVER, &IID_IClassFactory, (void **)&cf);
+    ok_ole_success(hr, CoCreateInstance);
+
+    IClassFactory_Release(cf);
+
+    hr = CoCreateInstance(&CLSID_WineOOPTest, NULL, CLSCTX_LOCAL_SERVER, &IID_IClassFactory, (void **)&cf);
+    ok(hr == REGDB_E_CLASSNOTREG, "Second CoCreateInstance on REGCLS_SINGLEUSE object should have failed\n");
+
+    quit_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Quit Event");
+    SetEvent(quit_event);
+
+    WaitForSingleObject(process, INFINITE);
+    CloseHandle(quit_event);
+    CloseHandle(process);
 }
 
 struct git_params
@@ -2374,9 +2563,7 @@ static void WINAPI TestChannelHook_ClientGetSize(
     trace("\t%s method %d\n", debugstr_iid(riid), info->iMethod);
     trace("\tcid: %s\n", debugstr_iid(&info->uCausality));
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
-    }
     ok(!info->pObject, "info->pObject should be NULL\n");
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
 
@@ -2393,9 +2580,7 @@ static void WINAPI TestChannelHook_ClientFillBuffer(
     SChannelHookCallInfo *info = (SChannelHookCallInfo *)riid;
     trace("TestChannelHook_ClientFillBuffer\n");
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
-    }
     ok(!info->pObject, "info->pObject should be NULL\n");
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
 
@@ -2415,8 +2600,8 @@ static void WINAPI TestChannelHook_ClientNotify(
     SChannelHookCallInfo *info = (SChannelHookCallInfo *)riid;
     trace("TestChannelHook_ClientNotify hrFault = 0x%08x\n", hrFault);
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
+    todo_wine {
     ok(info->pObject != NULL, "info->pObject shouldn't be NULL\n");
     }
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
@@ -2535,8 +2720,21 @@ START_TEST(marshal)
 {
     WNDCLASS wndclass;
     HMODULE hOle32 = GetModuleHandle("ole32");
+    int argc;
+    char **argv;
+
     if (!(pCoInitializeEx = (void*)GetProcAddress(hOle32, "CoInitializeEx"))) goto no_test;
 
+    argc = winetest_get_mainargs( &argv );
+    if (argc > 2 && (!strcmp(argv[2], "-Embedding")))
+    {
+        pCoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
+        test_register_local_server();
+        CoUninitialize();
+
+        return;
+    }
+
     /* register a window class used in several tests */
     memset(&wndclass, 0, sizeof(wndclass));
     wndclass.lpfnWndProc = window_proc;
@@ -2583,9 +2781,9 @@ START_TEST(marshal)
     test_freethreadedmarshaler();
     test_inproc_handler();
     test_handler_marshaling();
+    test_client_security();
 
-    /* doesn't pass with Win9x COM DLLs (even though Essential COM says it should) */
-    if (0) test_out_of_process_com();
+    test_local_server();
 
     test_globalinterfacetable();
 
