@@ -105,6 +105,7 @@ IUnknown * CALLBACK QCAP_createCaptureGraphBuilder2(IUnknown *pUnkOuter,
         pCapture->ref = 1;
         pCapture->mygraph = NULL;
         InitializeCriticalSection(&pCapture->csFilter);
+        pCapture->csFilter.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": CaptureGraphImpl.csFilter");
         *phr = S_OK;
         ObjectRefCount(TRUE);
     }
@@ -145,7 +146,7 @@ fnCaptureGraphBuilder2_AddRef(ICaptureGraphBuilder2 * iface)
     CaptureGraphImpl *This = impl_from_ICaptureGraphBuilder2(iface);
     DWORD ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p/%p)->() AddRef from %ld\n", This, iface, ref - 1);
+    TRACE("(%p/%p)->() AddRef from %d\n", This, iface, ref - 1);
     return ref;
 }
 
@@ -155,11 +156,12 @@ fnCaptureGraphBuilder2_Release(ICaptureGraphBuilder2 * iface)
     CaptureGraphImpl *This = impl_from_ICaptureGraphBuilder2(iface);
     DWORD ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p/%p)->() Release from %ld\n", This, iface, ref + 1);
+    TRACE("(%p/%p)->() Release from %d\n", This, iface, ref + 1);
 
     if (!ref)
     {
         FIXME("Release IGraphFilter or w/e\n");
+        This->csFilter.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->csFilter);
         This->lpVtbl = NULL;
         This->lpVtbl2 = NULL;
@@ -303,8 +305,8 @@ fnCaptureGraphBuilder2_AllocCapFile(ICaptureGraphBuilder2 * iface,
 {
     CaptureGraphImpl *This = impl_from_ICaptureGraphBuilder2(iface);
 
-    FIXME("(%p/%p)->(%s, %lld) Stub!\n", This, iface,
-          debugstr_w(lpwstr), dwlSize);
+    FIXME("(%p/%p)->(%s, 0x%s) Stub!\n", This, iface,
+          debugstr_w(lpwstr), wine_dbgstr_longlong(dwlSize));
 
     return E_NOTIMPL;
 }
