@@ -71,6 +71,7 @@ HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMP
     pParser->lpVtbl = &Parser_Vtbl;
     pParser->refCount = 1;
     InitializeCriticalSection(&pParser->csFilter);
+    pParser->csFilter.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": ParserImpl.csFilter");
     pParser->state = State_Stopped;
     pParser->pClock = NULL;
     ZeroMemory(&pParser->filterInfo, sizeof(FILTER_INFO));
@@ -93,6 +94,7 @@ HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMP
     else
     {
         CoTaskMemFree(pParser->ppPins);
+        pParser->csFilter.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&pParser->csFilter);
         CoTaskMemFree(pParser);
     }
@@ -169,7 +171,7 @@ static ULONG WINAPI Parser_AddRef(IBaseFilter * iface)
     ParserImpl *This = (ParserImpl *)iface;
     ULONG refCount = InterlockedIncrement(&This->refCount);
 
-    TRACE("(%p/%p)->() AddRef from %ld\n", This, iface, refCount - 1);
+    TRACE("(%p/%p)->() AddRef from %d\n", This, iface, refCount - 1);
 
     return refCount;
 }
@@ -179,23 +181,36 @@ static ULONG WINAPI Parser_Release(IBaseFilter * iface)
     ParserImpl *This = (ParserImpl *)iface;
     ULONG refCount = InterlockedDecrement(&This->refCount);
 
-    TRACE("(%p/%p)->() Release from %ld\n", This, iface, refCount + 1);
+    TRACE("(%p)->() Release from %d\n", This, refCount + 1);
     
     if (!refCount)
     {
         ULONG i;
 
-        DeleteCriticalSection(&This->csFilter);
         if (This->pClock)
             IReferenceClock_Release(This->pClock);
         
         for (i = 0; i < This->cStreams + 1; i++)
+        {
+            IPin *pConnectedTo;
+
+            if (SUCCEEDED(IPin_ConnectedTo(This->ppPins[i], &pConnectedTo)))
+            {
+                IPin_Disconnect(pConnectedTo);
+                IPin_Release(pConnectedTo);
+            }
+            IPin_Disconnect(This->ppPins[i]);
+
             IPin_Release(This->ppPins[i]);
+        }
         
-        HeapFree(GetProcessHeap(), 0, This->ppPins);
+        CoTaskMemFree(This->ppPins);
         This->lpVtbl = NULL;
+
+        This->csFilter.DebugInfo->Spare[0] = 0;
+        DeleteCriticalSection(&This->csFilter);
         
-        TRACE("Destroying AVI splitter\n");
+        TRACE("Destroying parser\n");
         CoTaskMemFree(This);
         
         return 0;
@@ -208,9 +223,11 @@ static ULONG WINAPI Parser_Release(IBaseFilter * iface)
 
 static HRESULT WINAPI Parser_GetClassID(IBaseFilter * iface, CLSID * pClsid)
 {
+    ParserImpl *This = (ParserImpl *)iface;
+
     TRACE("(%p)\n", pClsid);
 
-    *pClsid = CLSID_AviSplitter;
+    *pClsid = This->clsid;
 
     return S_OK;
 }
@@ -342,7 +359,7 @@ static HRESULT WINAPI Parser_GetState(IBaseFilter * iface, DWORD dwMilliSecsTime
 {
     ParserImpl *This = (ParserImpl *)iface;
 
-    TRACE("(%ld, %p)\n", dwMilliSecsTimeout, pState);
+    TRACE("(%d, %p)\n", dwMilliSecsTimeout, pState);
 
     EnterCriticalSection(&This->csFilter);
     {
@@ -486,7 +503,7 @@ HRESULT Parser_AddPin(ParserImpl * This, PIN_INFO * piOutput, ALLOCATOR_PROPERTI
 
     ppOldPins = This->ppPins;
 
-    This->ppPins = HeapAlloc(GetProcessHeap(), 0, (This->cStreams + 2) * sizeof(IPin *));
+    This->ppPins = CoTaskMemAlloc((This->cStreams + 2) * sizeof(IPin *));
     memcpy(This->ppPins, ppOldPins, (This->cStreams + 1) * sizeof(IPin *));
 
     hr = Parser_OutputPin_Construct(piOutput, props, NULL, Parser_OutputPin_QueryAccept, amt, fSamplesPerSec, &This->csFilter, This->ppPins + This->cStreams + 1);
@@ -497,13 +514,13 @@ HRESULT Parser_AddPin(ParserImpl * This, PIN_INFO * piOutput, ALLOCATOR_PROPERTI
         ((Parser_OutputPin *)(This->ppPins[This->cStreams + 1]))->dwLength = dwLength;
         ((Parser_OutputPin *)(This->ppPins[This->cStreams + 1]))->pin.pin.pUserData = (LPVOID)This->ppPins[This->cStreams + 1];
         This->cStreams++;
-        HeapFree(GetProcessHeap(), 0, ppOldPins);
+        CoTaskMemFree(ppOldPins);
     }
     else
     {
-        HeapFree(GetProcessHeap(), 0, This->ppPins);
+        CoTaskMemFree(This->ppPins);
         This->ppPins = ppOldPins;
-        ERR("Failed with error %lx\n", hr);
+        ERR("Failed with error %x\n", hr);
     }
 
     return hr;
@@ -517,7 +534,7 @@ static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
     IPin ** ppOldPins = This->ppPins;
 
     /* reduce the pin array down to 1 (just our input pin) */
-    This->ppPins = HeapAlloc(GetProcessHeap(), 0, sizeof(IPin *) * 1);
+    This->ppPins = CoTaskMemAlloc(sizeof(IPin *) * 1);
     memcpy(This->ppPins, ppOldPins, sizeof(IPin *) * 1);
 
     for (i = 0; i < This->cStreams; i++)
@@ -527,7 +544,7 @@ static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
     }
 
     This->cStreams = 0;
-    HeapFree(GetProcessHeap(), 0, ppOldPins);
+    CoTaskMemFree(ppOldPins);
 
     return S_OK;
 }
@@ -596,7 +613,7 @@ static const IMediaSeekingVtbl Parser_Seeking_Vtbl =
     MediaSeekingImpl_GetPreroll
 };
 
-HRESULT WINAPI Parser_OutputPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
+static HRESULT WINAPI Parser_OutputPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
 {
     Parser_OutputPin *This = (Parser_OutputPin *)iface;
 
@@ -627,7 +644,7 @@ static ULONG WINAPI Parser_OutputPin_Release(IPin * iface)
     Parser_OutputPin *This = (Parser_OutputPin *)iface;
     ULONG refCount = InterlockedDecrement(&This->pin.pin.refCount);
     
-    TRACE("(%p)->() Release from %ld\n", iface, refCount + 1);
+    TRACE("(%p)->() Release from %d\n", iface, refCount + 1);
     
     if (!refCount)
     {
