@@ -877,3 +877,120 @@ void WINAPI AssertFail(LPCSTR lpFile, UINT uLine, LPCSTR lpMessage)
 {
     FIXME("%s %u %s\n", lpFile, uLine, lpMessage);
 }
+
+/***********************************************************************
+ *      SetupCopyOEMInfA  (SETUPAPI.@)
+ */
+BOOL WINAPI SetupCopyOEMInfA( PCSTR source, PCSTR location,
+                              DWORD media_type, DWORD style, PSTR dest,
+                              DWORD buffer_size, PDWORD required_size, PSTR *component )
+{
+    BOOL ret = FALSE;
+    LPWSTR destW = NULL, sourceW = NULL, locationW = NULL;
+    DWORD size;
+
+    TRACE("%s, %s, %d, %d, %p, %d, %p, %p\n", debugstr_a(source), debugstr_a(location),
+          media_type, style, dest, buffer_size, required_size, component);
+
+    if (dest && !(destW = MyMalloc( buffer_size * sizeof(WCHAR) ))) return FALSE;
+    if (source && !(sourceW = strdupAtoW( source ))) goto done;
+    if (location && !(locationW = strdupAtoW( location ))) goto done;
+
+    if (!(ret = SetupCopyOEMInfW( sourceW, locationW, media_type, style, destW,
+                                  buffer_size, &size, NULL )))
+    {
+        if (required_size) *required_size = size;
+        goto done;
+    }
+
+    if (dest)
+    {
+        if (buffer_size >= size)
+        {
+            WideCharToMultiByte( CP_ACP, 0, destW, -1, dest, buffer_size, NULL, NULL );
+            if (component) *component = strrchr( dest, '\\' ) + 1;
+        }
+        else
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            goto done;
+        }
+    }
+
+done:
+    MyFree( destW );
+    HeapFree( GetProcessHeap(), 0, sourceW );
+    HeapFree( GetProcessHeap(), 0, locationW );
+    if (ret) SetLastError(ERROR_SUCCESS);
+    return ret;
+}
+
+/***********************************************************************
+ *      SetupCopyOEMInfW  (SETUPAPI.@)
+ */
+BOOL WINAPI SetupCopyOEMInfW( PCWSTR source, PCWSTR location,
+                              DWORD media_type, DWORD style, PWSTR dest,
+                              DWORD buffer_size, PDWORD required_size, PWSTR *component )
+{
+    BOOL ret = FALSE;
+    WCHAR target[MAX_PATH], *p;
+    static const WCHAR inf_oem[] = { '\\','i','n','f','\\','O','E','M',0 };
+    DWORD size;
+
+    TRACE("%s, %s, %d, %d, %p, %d, %p, %p\n", debugstr_w(source), debugstr_w(location),
+          media_type, style, dest, buffer_size, required_size, component);
+
+    if (!source)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
+
+    /* check for a relative path */
+    if (!(*source == '\\' || (*source && source[1] == ':')))
+    {
+        SetLastError(ERROR_FILE_NOT_FOUND);
+        return FALSE;
+    }
+
+    if (!GetWindowsDirectoryW( target, sizeof(target)/sizeof(WCHAR) )) return FALSE;
+
+    strcatW( target, inf_oem );
+    if ((p = strrchrW( source, '\\' )))
+        strcatW( target, p + 1 );
+
+    if (!(ret = CopyFileW( source, target, (style & SP_COPY_NOOVERWRITE) != 0 )))
+        return ret;
+
+    if (style & SP_COPY_DELETESOURCE)
+        DeleteFileW( source );
+
+    size = strlenW( target ) + 1;
+    if (dest)
+    {
+        if (buffer_size >= size)
+        {
+            strcpyW( dest, target );
+        }
+        else
+        {
+            SetLastError( ERROR_INSUFFICIENT_BUFFER );
+            ret = FALSE;
+        }
+    }
+
+    if (component) *component = p + 1;
+    if (required_size) *required_size = size;
+    if (ret) SetLastError(ERROR_SUCCESS);
+
+    return ret;
+}
+
+/***********************************************************************
+ *      InstallCatalog  (SETUPAPI.@)
+ */
+DWORD WINAPI InstallCatalog( LPCSTR catalog, LPCSTR basename, LPSTR fullname )
+{
+    FIXME("%s, %s, %p\n", debugstr_a(catalog), debugstr_a(basename), fullname);
+    return 0;
+}
