@@ -174,7 +174,7 @@ static ULONG WINAPI IAutoComplete_fnAddRef(
     IAutoCompleteImpl *This = (IAutoCompleteImpl *)iface;
     ULONG refCount = InterlockedIncrement(&This->ref);
     
-    TRACE("(%p)->(%lu)\n", This, refCount - 1);
+    TRACE("(%p)->(%u)\n", This, refCount - 1);
 
     return refCount;
 }
@@ -188,7 +188,7 @@ static ULONG WINAPI IAutoComplete_fnRelease(
     IAutoCompleteImpl *This = (IAutoCompleteImpl *)iface;
     ULONG refCount = InterlockedDecrement(&This->ref);
     
-    TRACE("(%p)->(%lu)\n", This, refCount + 1);
+    TRACE("(%p)->(%u)\n", This, refCount + 1);
 
     if (!refCount) {
 	TRACE(" destroying IAutoComplete(%p)\n",This);
@@ -347,7 +347,7 @@ static ULONG WINAPI IAutoComplete2_fnAddRef(
 {
     IAutoCompleteImpl *This = impl_from_IAutoComplete2(iface);
 
-    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
+    TRACE ("(%p)->(count=%u)\n", This, This->ref);
 
     return IAutoComplete2_AddRef((IAutoComplete*)This);
 }
@@ -360,7 +360,7 @@ static ULONG WINAPI IAutoComplete2_fnRelease(
 {
     IAutoCompleteImpl *This = impl_from_IAutoComplete2(iface);
 
-    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
+    TRACE ("(%p)->(count=%u)\n", This, This->ref);
 
     return IAutoComplete_Release((IAutoComplete*)This);
 }
@@ -425,7 +425,7 @@ static HRESULT WINAPI IAutoComplete2_fnSetOptions(
 
     IAutoCompleteImpl *This = impl_from_IAutoComplete2(iface);
 
-    TRACE("(%p) -> (0x%lx)\n", This, dwFlag);
+    TRACE("(%p) -> (0x%x)\n", This, dwFlag);
 
     This->options = dwFlag;
 
@@ -624,7 +624,7 @@ static LRESULT APIENTRY ACLBoxSubclassProc(HWND hwnd, UINT uMsg, WPARAM wParam,
 {
     IAutoCompleteImpl *This = (IAutoCompleteImpl *)GetWindowLongPtrW(hwnd, GWLP_USERDATA);
     WCHAR *msg;
-    int sel = -1, len;
+    int sel, len;
 
     switch (uMsg) {
 	case WM_MOUSEMOVE:
@@ -632,9 +632,11 @@ static LRESULT APIENTRY ACLBoxSubclassProc(HWND hwnd, UINT uMsg, WPARAM wParam,
 	    SendMessageW(hwnd, LB_SETCURSEL, (WPARAM)sel, (LPARAM)0);
 	    break;
 	case WM_LBUTTONDOWN:
-	    len = SendMessageW(This->hwndListBox, LB_GETTEXTLEN, sel, (LPARAM)NULL);
+	    sel = SendMessageW(hwnd, LB_GETCURSEL, 0, 0);
+	    if (sel < 0)
+		break;
+	    len = SendMessageW(This->hwndListBox, LB_GETTEXTLEN, sel, 0);
 	    msg = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (len+1)*sizeof(WCHAR));
-	    sel = (INT)SendMessageW(hwnd, LB_GETCURSEL, 0, 0);
 	    SendMessageW(hwnd, LB_GETTEXT, sel, (LPARAM)msg);
 	    SendMessageW(This->hwndEdit, WM_SETTEXT, 0, (LPARAM)msg);
 	    SendMessageW(This->hwndEdit, EM_SETSEL, 0, lstrlenW(msg));
