@@ -31,7 +31,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3d_shader);
 
 #define GLNAME_REQUIRE_GLSL  ((const char *)1)
 
-inline static BOOL shader_is_version_token(DWORD token) {
+static inline BOOL shader_is_version_token(DWORD token) {
     return shader_is_pshader_version(token) ||
            shader_is_vshader_version(token);
 }
@@ -57,9 +57,18 @@ int shader_addline(
         return -1;
     }
 
+    if (buffer->newline) {
+        TRACE("GL HW (%u, %u) : %s", buffer->lineNo + 1, buffer->bsize, base);
+        buffer->newline = FALSE;
+    } else {
+        TRACE("%s", base);
+    }
+
     buffer->bsize += rc;
-    buffer->lineNo++;
-    TRACE("GL HW (%u, %u) : %s", buffer->lineNo, buffer->bsize, base); 
+    if (buffer->buffer[buffer->bsize-1] == '\n') {
+        buffer->lineNo++;
+        buffer->newline = TRUE;
+    }
     return 0;
 }
 
@@ -188,6 +197,8 @@ HRESULT shader_get_registers_used(
     /* There are some minor differences between pixel and vertex shaders */
     char pshader = shader_is_pshader_version(This->baseShader.hex_version);
 
+    reg_maps->bumpmat = -1;
+
     if (pToken == NULL)
         return WINED3D_OK;
 
@@ -286,7 +297,15 @@ HRESULT shader_get_registers_used(
             DWORD snum = *pToken & WINED3DSP_REGNUM_MASK; 
             reg_maps->labels[snum] = 1;
             pToken += curOpcode->num_params;
- 
+
+        } else if(WINED3DSIO_BEM == curOpcode->opcode) {
+            DWORD regnum = *pToken & WINED3DSP_REGNUM_MASK;
+            if(reg_maps->bumpmat != -1 && reg_maps->bumpmat != regnum) {
+                FIXME("Pixel shader uses bem or texbem instruction on more than 1 sampler\n");
+            } else {
+                reg_maps->bumpmat = regnum;
+            }
+
         /* Set texture, address, temporary registers */
         } else {
             int i, limit;
@@ -314,10 +333,6 @@ HRESULT shader_get_registers_used(
                 } else {
                     int texType = IWineD3DBaseTexture_GetTextureDimensions(stateBlock->textures[sampler_code]);
                     switch(texType) {
-                        case GL_TEXTURE_1D:
-                            reg_maps->samplers[sampler_code] = (0x1 << 31) | WINED3DSTT_1D;
-                            break;
-
                         case GL_TEXTURE_2D:
                             reg_maps->samplers[sampler_code] = (0x1 << 31) | WINED3DSTT_2D;
                             break;
@@ -335,6 +350,15 @@ HRESULT shader_get_registers_used(
                             reg_maps->samplers[sampler_code] = (0x1 << 31) | WINED3DSTT_2D;
                     }
                 }
+
+                /* texbem is only valid with < 1.4 pixel shaders */
+                if(WINED3DSIO_TEXBEM == curOpcode->opcode) {
+                    if(reg_maps->bumpmat != -1 && reg_maps->bumpmat != sampler_code) {
+                        FIXME("Pixel shader uses texbem instruction on more than 1 sampler\n");
+                    } else {
+                        reg_maps->bumpmat = sampler_code;
+                    }
+                }
             }
 
             /* This will loop over all the registers and try to
@@ -410,50 +434,50 @@ static void shader_dump_decl_usage(
             TRACE("_");
 
         switch(usage) {
-        case D3DDECLUSAGE_POSITION:
+        case WINED3DDECLUSAGE_POSITION:
             TRACE("position%d", idx);
             break;
-        case D3DDECLUSAGE_BLENDINDICES:
+        case WINED3DDECLUSAGE_BLENDINDICES:
             TRACE("blend");
             break;
-        case D3DDECLUSAGE_BLENDWEIGHT:
+        case WINED3DDECLUSAGE_BLENDWEIGHT:
             TRACE("weight");
             break;
-        case D3DDECLUSAGE_NORMAL:
+        case WINED3DDECLUSAGE_NORMAL:
             TRACE("normal%d", idx);
             break;
-        case D3DDECLUSAGE_PSIZE:
+        case WINED3DDECLUSAGE_PSIZE:
             TRACE("psize");
             break;
-        case D3DDECLUSAGE_COLOR:
+        case WINED3DDECLUSAGE_COLOR:
             if(idx == 0)  {
                 TRACE("color");
             } else {
                 TRACE("specular%d", (idx - 1));
             }
             break;
-        case D3DDECLUSAGE_TEXCOORD:
+        case WINED3DDECLUSAGE_TEXCOORD:
             TRACE("texture%d", idx);
             break;
-        case D3DDECLUSAGE_TANGENT:
+        case WINED3DDECLUSAGE_TANGENT:
             TRACE("tangent");
             break;
-        case D3DDECLUSAGE_BINORMAL:
+        case WINED3DDECLUSAGE_BINORMAL:
             TRACE("binormal");
             break;
-        case D3DDECLUSAGE_TESSFACTOR:
+        case WINED3DDECLUSAGE_TESSFACTOR:
             TRACE("tessfactor");
             break;
-        case D3DDECLUSAGE_POSITIONT:
+        case WINED3DDECLUSAGE_POSITIONT:
             TRACE("positionT%d", idx);
             break;
-        case D3DDECLUSAGE_FOG:
+        case WINED3DDECLUSAGE_FOG:
             TRACE("fog");
             break;
-        case D3DDECLUSAGE_DEPTH:
+        case WINED3DDECLUSAGE_DEPTH:
             TRACE("depth");
             break;
-        case D3DDECLUSAGE_SAMPLE:
+        case WINED3DDECLUSAGE_SAMPLE:
             TRACE("sample");
             break;
         default:
@@ -913,6 +937,9 @@ void shader_trace_init(
                             default:
                                 TRACE("_(%u)", op);
                         }
+                    } else if (curOpcode->opcode == WINED3DSIO_TEX &&
+                               This->baseShader.hex_version >= WINED3DPS_VERSION(2,0)) {
+                        if(opcode_token & WINED3DSI_TEXLD_PROJECT) TRACE("p");
                     }
 
                     /* Destination token */
@@ -971,7 +998,7 @@ void shader_delete_constant_list(
 static void shader_none_select(IWineD3DDevice *iface, BOOL usePS, BOOL useVS) {}
 static void shader_none_select_depth_blt(IWineD3DDevice *iface) {}
 static void shader_none_load_constants(IWineD3DDevice *iface, char usePS, char useVS) {}
-static void shader_none_cleanup(BOOL usePS, BOOL useVS) {}
+static void shader_none_cleanup(IWineD3DDevice *iface) {}
 
 const shader_backend_t none_shader_backend = {
     &shader_none_select,
