@@ -152,7 +152,6 @@ static WINED3DGLTYPE const glTypeLookup[WINED3DDECLTYPE_UNUSED] = {
  */
 #define VS_NONE    0
 #define VS_HW      1
-#define VS_SW      2
 
 #define PS_NONE    0
 #define PS_HW      1
@@ -168,7 +167,6 @@ static WINED3DGLTYPE const glTypeLookup[WINED3DDECLTYPE_UNUSED] = {
 #define ORM_PBUFFER     1
 #define ORM_FBO         2
 
-#define SHADER_SW   0
 #define SHADER_ARB  1
 #define SHADER_GLSL 2
 #define SHADER_NONE 3
@@ -429,6 +427,8 @@ void primitiveConvertFVFtoOffset(DWORD thisFVF,
 
 DWORD get_flexible_vertex_size(DWORD d3dvtVertexType);
 
+void blt_to_drawable(IWineD3DDeviceImpl *This, IWineD3DSurfaceImpl *surface);
+
 #define eps 1e-8
 
 #define GET_TEXCOORD_SIZE_FROM_FVF(d3dvtVertexType, tex_num) \
@@ -506,7 +506,7 @@ struct WineD3DContext {
     DWORD                   isStateDirty[STATE_HIGHEST/32 + 1]; /* Bitmap to find out quickly if a state is dirty */
 
     IWineD3DSurface         *surface;
-    /* TODO: Thread this ctx belongs to                    */
+    DWORD                   tid;    /* Thread ID which owns this context at the moment */
 
     /* Stores some inforation about the context state for optimization */
     BOOL                    last_was_rhw;      /* true iff last draw_primitive was in xyzrhw mode */
@@ -1023,7 +1023,6 @@ struct IWineD3DSurfaceImpl
 
     UINT                      pow2Width;
     UINT                      pow2Height;
-    UINT                      pow2Size;
 
     /* Oversized texture */
     RECT                      glRect;
@@ -1084,7 +1083,6 @@ HRESULT WINAPI IWineD3DSurfaceImpl_SetColorKey(IWineD3DSurface *iface, DWORD Fla
 HRESULT WINAPI IWineD3DSurfaceImpl_CleanDirtyRect(IWineD3DSurface *iface);
 extern HRESULT WINAPI IWineD3DSurfaceImpl_AddDirtyRect(IWineD3DSurface *iface, CONST RECT* pDirtyRect);
 HRESULT WINAPI IWineD3DSurfaceImpl_SetContainer(IWineD3DSurface *iface, IWineD3DBase *container);
-HRESULT WINAPI IWineD3DSurfaceImpl_SetPBufferState(IWineD3DSurface *iface, BOOL inPBuffer, BOOL  inTexture);
 void WINAPI IWineD3DSurfaceImpl_SetGlTextureDesc(IWineD3DSurface *iface, UINT textureName, int target);
 void WINAPI IWineD3DSurfaceImpl_GetGlDesc(IWineD3DSurface *iface, glDescriptor **glDescription);
 const void *WINAPI IWineD3DSurfaceImpl_GetData(IWineD3DSurface *iface);
@@ -1106,27 +1104,24 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UpdateOverlay(IWineD3DSurface *iface, RECT *S
 #define SFLAG_OVERSIZE    0x00000001 /* Surface is bigger than gl size, blts only */
 #define SFLAG_CONVERTED   0x00000002 /* Converted for color keying or Palettized */
 #define SFLAG_DIBSECTION  0x00000004 /* Has a DIB section attached for getdc */
-#define SFLAG_DIRTY       0x00000008 /* Surface was locked by the app */
-#define SFLAG_LOCKABLE    0x00000010 /* Surface can be locked */
-#define SFLAG_DISCARD     0x00000020 /* ??? */
-#define SFLAG_LOCKED      0x00000040 /* Surface is locked atm */
-#define SFLAG_INTEXTURE   0x00000080 /* ??? */
-#define SFLAG_INPBUFFER   0x00000100 /* ??? */
+#define SFLAG_LOCKABLE    0x00000008 /* Surface can be locked */
+#define SFLAG_DISCARD     0x00000010 /* ??? */
+#define SFLAG_LOCKED      0x00000020 /* Surface is locked atm */
+#define SFLAG_INTEXTURE   0x00000040 /* The GL texture contains the newest surface content */
+#define SFLAG_INDRAWABLE  0x00000080 /* The gl drawable contains the most up to date data */
+#define SFLAG_INSYSMEM    0x00000100 /* The system memory copy is most up to date */
 #define SFLAG_NONPOW2     0x00000200 /* Surface sizes are not a power of 2 */
 #define SFLAG_DYNLOCK     0x00000400 /* Surface is often locked by the app */
 #define SFLAG_DYNCHANGE   0x00000C00 /* Surface contents are changed very often, implies DYNLOCK */
 #define SFLAG_DCINUSE     0x00001000 /* Set between GetDC and ReleaseDC calls */
-#define SFLAG_GLDIRTY     0x00002000 /* The opengl texture is more up to date than the surface mem */
-#define SFLAG_LOST        0x00004000 /* Surface lost flag for DDraw */
-#define SFLAG_FORCELOAD   0x00008000 /* To force PreLoading of a scratch cursor */
-#define SFLAG_USERPTR     0x00010000 /* The application allocated the memory for this surface */
-#define SFLAG_GLCKEY      0x00020000 /* The gl texture was created with a color key */
+#define SFLAG_LOST        0x00002000 /* Surface lost flag for DDraw */
+#define SFLAG_USERPTR     0x00004000 /* The application allocated the memory for this surface */
+#define SFLAG_GLCKEY      0x00008000 /* The gl texture was created with a color key */
 
 /* In some conditions the surface memory must not be freed:
  * SFLAG_OVERSIZE: Not all data can be kept in GL
  * SFLAG_CONVERTED: Converting the data back would take too long
  * SFLAG_DIBSECTION: The dib code manages the memory
- * SFLAG_DIRTY: GL surface isn't up to date
  * SFLAG_LOCKED: The app requires access to the surface data
  * SFLAG_DYNLOCK: Avoid freeing the data for performance
  * SFLAG_DYNCHANGE: Same reason as DYNLOCK
@@ -1134,7 +1129,6 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UpdateOverlay(IWineD3DSurface *iface, RECT *S
 #define SFLAG_DONOTFREE  (SFLAG_OVERSIZE   | \
                           SFLAG_CONVERTED  | \
                           SFLAG_DIBSECTION | \
-                          SFLAG_DIRTY      | \
                           SFLAG_LOCKED     | \
                           SFLAG_DYNLOCK    | \
                           SFLAG_DYNCHANGE  | \
@@ -1149,18 +1143,15 @@ BOOL CalculateTexRect(IWineD3DSurfaceImpl *This, RECT *Rect, float glTexCoord[4]
  * IWineD3DVertexDeclaration implementation structure
  */
 typedef struct IWineD3DVertexDeclarationImpl {
- /* IUnknown  Information     */
-  const IWineD3DVertexDeclarationVtbl *lpVtbl;
-  LONG                    ref;     /* Note: Ref counting not required */
+    /* IUnknown  Information */
+    const IWineD3DVertexDeclarationVtbl *lpVtbl;
+    LONG                    ref;
 
-  IUnknown               *parent;
-  /** precomputed fvf if simple declaration */
-  IWineD3DDeviceImpl     *wineD3DDevice;
-  DWORD   fvf[MAX_STREAMS];
-  DWORD   allFVF;
+    IUnknown                *parent;
+    IWineD3DDeviceImpl      *wineD3DDevice;
 
-  WINED3DVERTEXELEMENT  *pDeclarationWine;
-  UINT                   declarationWNumElements;
+    WINED3DVERTEXELEMENT    *pDeclarationWine;
+    UINT                    declarationWNumElements;
 } IWineD3DVertexDeclarationImpl;
 
 extern const IWineD3DVertexDeclarationVtbl IWineD3DVertexDeclaration_Vtbl;
@@ -1366,7 +1357,8 @@ typedef struct IWineD3DSwapChainImpl
 
     long prev_time, frames;   /* Performance tracking */
 
-    WineD3DContext         *context; /* Later a array for multithreading */
+    WineD3DContext        **context; /* Later a array for multithreading */
+    unsigned int            num_contexts;
 
     HWND                    win_handle;
     Window                  win;
@@ -1461,7 +1453,6 @@ void multiply_matrix(WINED3DMATRIX *dest, const WINED3DMATRIX *src1, const WINED
     void IWineD3DBaseTextureImpl_CleanUp(IWineD3DBaseTexture *iface);
 
 struct SHADER_OPCODE_ARG;
-typedef void (*shader_fct_t)();
 typedef void (*SHADER_HANDLER) (struct SHADER_OPCODE_ARG*);
 
 /* Struct to maintain a list of GLSL shader programs and their associated pixel and
@@ -1557,7 +1548,6 @@ typedef struct SHADER_OPCODE {
     const char*   glname;
     char          dst_token;
     CONST UINT    num_params;
-    shader_fct_t  soft_fct;
     SHADER_HANDLER hw_fct;
     SHADER_HANDLER hw_glsl_fct;
     DWORD         min_version;
@@ -1635,6 +1625,7 @@ extern void shader_arb_load_constants(
 extern void shader_hw_def(SHADER_OPCODE_ARG *arg);
 
 /* ARB pixel shader prototypes */
+extern void pshader_hw_bem(SHADER_OPCODE_ARG* arg);
 extern void pshader_hw_cnd(SHADER_OPCODE_ARG* arg);
 extern void pshader_hw_cmp(SHADER_OPCODE_ARG* arg);
 extern void pshader_hw_map2gl(SHADER_OPCODE_ARG* arg);
@@ -1710,6 +1701,7 @@ extern void pshader_glsl_texm3x3spec(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texm3x3vspec(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texkill(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texbem(SHADER_OPCODE_ARG* arg);
+extern void pshader_glsl_bem(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texreg2ar(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texreg2gb(SHADER_OPCODE_ARG* arg);
 extern void pshader_glsl_texreg2rgb(SHADER_OPCODE_ARG* arg);
@@ -1816,22 +1808,22 @@ extern void print_glsl_info_log(
     WineD3D_GL_Info *gl_info,
     GLhandleARB obj);
 
-inline static int shader_get_regtype(const DWORD param) {
+static inline int shader_get_regtype(const DWORD param) {
     return (((param & WINED3DSP_REGTYPE_MASK) >> WINED3DSP_REGTYPE_SHIFT) |
             ((param & WINED3DSP_REGTYPE_MASK2) >> WINED3DSP_REGTYPE_SHIFT2));
 }
 
 extern unsigned int shader_get_float_offset(const DWORD reg);
 
-inline static BOOL shader_is_pshader_version(DWORD token) {
+static inline BOOL shader_is_pshader_version(DWORD token) {
     return 0xFFFF0000 == (token & 0xFFFF0000);
 }
 
-inline static BOOL shader_is_vshader_version(DWORD token) {
+static inline BOOL shader_is_vshader_version(DWORD token) {
     return 0xFFFE0000 == (token & 0xFFFF0000);
 }
 
-inline static BOOL shader_is_comment(DWORD token) {
+static inline BOOL shader_is_comment(DWORD token) {
     return WINED3DSIO_COMMENT == (token & WINED3DSI_OPCODE_MASK);
 }
 
@@ -1945,4 +1937,18 @@ typedef struct {
 } PixelFormatDesc;
 
 const PixelFormatDesc *getFormatDescEntry(WINED3DFORMAT fmt);
+
+static inline BOOL use_vs(IWineD3DDeviceImpl *device) {
+    return (device->vs_selected_mode != SHADER_NONE
+            && device->stateBlock->vertexShader
+            && ((IWineD3DVertexShaderImpl *)device->stateBlock->vertexShader)->baseShader.function
+            && !device->strided_streams.u.s.position_transformed);
+}
+
+static inline BOOL use_ps(IWineD3DDeviceImpl *device) {
+    return (device->ps_selected_mode != SHADER_NONE
+            && device->stateBlock->pixelShader
+            && ((IWineD3DPixelShaderImpl *)device->stateBlock->pixelShader)->baseShader.function);
+}
+
 #endif
