@@ -69,6 +69,7 @@ Visual *visual;
 unsigned int screen_width;
 unsigned int screen_height;
 unsigned int screen_depth;
+RECT virtual_screen_rect;
 Window root_window;
 int dxgrab = 0;
 int usexvidmode = 1;
@@ -78,6 +79,7 @@ int use_take_focus = 1;
 int use_primary_selection = 0;
 int managed_mode = 1;
 int private_color_map = 0;
+int primary_monitor = 0;
 int client_side_with_core = 1;
 int client_side_with_render = 1;
 int client_side_antialias_with_core = 1;
@@ -87,8 +89,6 @@ int alloc_system_colors = 256;
 DWORD thread_data_tls_index = TLS_OUT_OF_INDEXES;
 int xrender_error_base = 0;
 
-static BOOL desktop_dbl_buf = TRUE;
-
 static x11drv_error_callback err_callback;   /* current callback for error */
 static Display *err_callback_display;        /* display callback is set for */
 static void *err_callback_arg;               /* error callback argument */
@@ -274,7 +274,7 @@ void wine_tsx11_unlock(void)
  *
  * Get a config key from either the app-specific or the default config
  */
-inline static DWORD get_config_key( HKEY defkey, HKEY appkey, const char *name,
+static inline DWORD get_config_key( HKEY defkey, HKEY appkey, const char *name,
                                     char *buffer, DWORD size )
 {
     if (appkey && !RegQueryValueExA( appkey, name, 0, NULL, (LPBYTE)buffer, &size )) return 0;
@@ -349,15 +349,15 @@ static void setup_options(void)
     if (!get_config_key( hkey, appkey, "ClientSideAntiAliasWithRender", buffer, sizeof(buffer) ))
         client_side_antialias_with_render = IS_OPTION_TRUE( buffer[0] );
 
-    if (!get_config_key( hkey, appkey, "DesktopDoubleBuffered", buffer, sizeof(buffer) ))
-        desktop_dbl_buf = IS_OPTION_TRUE( buffer[0] );
-
     if (!get_config_key( hkey, appkey, "UseXIM", buffer, sizeof(buffer) ))
         use_xim = IS_OPTION_TRUE( buffer[0] );
 
     if (!get_config_key( hkey, appkey, "PrivateColorMap", buffer, sizeof(buffer) ))
         private_color_map = IS_OPTION_TRUE( buffer[0] );
 
+    if (!get_config_key( hkey, appkey, "PrimaryMonitor", buffer, sizeof(buffer) ))
+        primary_monitor = atoi( buffer );
+
     if (!get_config_key( hkey, appkey, "CopyDefaultColors", buffer, sizeof(buffer) ))
         copy_default_colors = atoi(buffer);
 
@@ -414,7 +414,7 @@ static BOOL process_attach(void)
     if (!screen_depth) screen_depth = DefaultDepthOfScreen( screen );
 
     /* If OpenGL is available, change the default visual, etc as necessary */
-    if (desktop_dbl_buf && (desktop_vi = X11DRV_setup_opengl_visual( display )))
+    if ((desktop_vi = X11DRV_setup_opengl_visual( display )))
     {
         visual       = desktop_vi->visual;
         screen       = ScreenOfDisplay(display, desktop_vi->screen);
@@ -429,6 +429,7 @@ static BOOL process_attach(void)
     screen_width  = WidthOfScreen( screen );
     screen_height = HeightOfScreen( screen );
 
+    xinerama_init();
     X11DRV_Settings_Init();
 
 #ifdef HAVE_LIBXXF86VM
@@ -440,6 +441,7 @@ static BOOL process_attach(void)
     X11DRV_XRandR_Init();
 #endif
 
+    X11DRV_ClipCursor( NULL );
     X11DRV_InitKeyboard();
     X11DRV_InitClipboard();
 
@@ -459,8 +461,8 @@ static void thread_detach(void)
         X11DRV_ResetSelectionOwner();
         CloseHandle( data->display_fd );
         wine_tsx11_lock();
+        if (data->xim) XCloseIM( data->xim );
         XCloseDisplay( data->display );
-        /* if (data->xim) XCloseIM( data->xim ); */ /* crashes Xlib */
         wine_tsx11_unlock();
         HeapFree( GetProcessHeap(), 0, data );
     }
@@ -514,14 +516,23 @@ struct x11drv_thread_data *x11drv_init_thread_data(void)
     if (use_xkb)
     {
         use_xkb = XkbUseExtension( data->display, NULL, NULL );
-        if (use_xkb) XkbSetDetectableAutoRepeat( data->display, True, NULL );
+        if (use_xkb)
+        {
+            /* Hack: dummy call to XkbKeysymToModifiers to force initialisation of Xkb internals */
+            /* This works around an Xlib bug where it tries to get the display lock */
+            /* twice during XFilterEvents if Xkb hasn't been initialised yet. */
+            XkbKeysymToModifiers( data->display, 'A' );
+            XkbSetDetectableAutoRepeat( data->display, True, NULL );
+        }
     }
 #endif
 
     if (TRACE_ON(synchronous)) XSynchronize( data->display, True );
     wine_tsx11_unlock();
 
-    if (use_xim && !(data->xim = X11DRV_SetupXIM( data->display, input_style )))
+    if (!use_xim)
+        data->xim = NULL;
+    else if (!(data->xim = X11DRV_SetupXIM( data->display, input_style )))
         WARN("Input Method is not available\n");
 
     if (wine_server_fd_to_handle( ConnectionNumber(data->display), GENERIC_READ | SYNCHRONIZE,
