@@ -177,12 +177,13 @@ static const struct object_ops dir_ops =
     dir_get_fd,               /* get_fd */
     dir_map_access,           /* map_access */
     no_lookup_name,           /* lookup_name */
-    no_close_handle,          /* close_handle */
+    no_open_file,             /* open_file */
+    fd_close_handle,          /* close_handle */
     dir_destroy               /* destroy */
 };
 
 static int dir_get_poll_events( struct fd *fd );
-static int dir_get_info( struct fd *fd );
+static enum server_fd_type dir_get_info( struct fd *fd, int *flags );
 static void dir_cancel_async( struct fd *fd );
 
 static const struct fd_ops dir_fd_ops =
@@ -282,10 +283,7 @@ void do_change_notify( int unix_fd )
 
 static void dir_signal_changed( struct dir *dir )
 {
-    if (dir->event)
-        set_event( dir->event );
-    else
-        wake_up( &dir->obj, 0 );
+    if (!dir->event) wake_up( &dir->obj, 0 );
 }
 
 /* SIGIO callback, called synchronously with the poll loop */
@@ -347,11 +345,7 @@ static void dir_destroy( struct object *obj )
     async_terminate_queue( &dir->change_q, STATUS_CANCELLED );
     while ((record = get_first_change_record( dir ))) free( record );
 
-    if (dir->event)
-    {
-        set_event( dir->event );
-        release_object( dir->event );
-    }
+    if (dir->event) release_object( dir->event );
     release_object( dir->fd );
 
     if (inotify_fd && list_empty( &change_list ))
@@ -372,9 +366,10 @@ static int dir_get_poll_events( struct fd *fd )
     return 0;
 }
 
-static int dir_get_info( struct fd *fd )
+static enum server_fd_type dir_get_info( struct fd *fd, int *flags )
 {
-    return 0;
+    *flags = 0;
+    return FD_TYPE_DIR;
 }
 
 static void dir_cancel_async( struct fd *fd )
@@ -550,14 +545,13 @@ static struct inode *inode_from_name( struct inode *inode, const char *name )
 
 static int inotify_get_poll_events( struct fd *fd );
 static void inotify_poll_event( struct fd *fd, int event );
-static int inotify_get_info( struct fd *fd );
 
 static const struct fd_ops inotify_fd_ops =
 {
     inotify_get_poll_events,  /* get_poll_events */
     inotify_poll_event,       /* poll_event */
     no_flush,                 /* flush */
-    inotify_get_info,         /* get_file_info */
+    no_get_file_info,         /* get_file_info */
     default_fd_queue_async,   /* queue_async */
     default_fd_cancel_async,  /* cancel_async */
 };
@@ -823,11 +817,6 @@ static void inotify_poll_event( struct fd *fd, int event )
     }
 }
 
-static int inotify_get_info( struct fd *fd )
-{
-    return 0;
-}
-
 static inline struct fd *create_inotify_fd( void )
 {
     int unix_fd;
@@ -1093,21 +1082,16 @@ DECL_HANDLER(read_directory_changes)
         return;
 
     /* possibly send changes through an event flag */
-    if (req->event)
-    {
-        event = get_event_obj( current->process, req->event, EVENT_MODIFY_STATE );
-        if (!event)
-            goto end;
-    }
+    if (req->async.event &&
+        !(event = get_event_obj( current->process, req->async.event, EVENT_MODIFY_STATE )))
+        goto end;
 
     /* discard the current data, and move onto the next event */
     if (dir->event) release_object( dir->event );
     dir->event = event;
 
     /* requests don't timeout */
-    if ( req->io_apc && !create_async( current, NULL, &dir->change_q,
-                        req->io_apc, req->io_user, req->io_sb ))
-        return;
+    if (!create_async( current, NULL, &dir->change_q, &req->async )) goto end;
 
     /* assign it once */
     if (!dir->filter)
@@ -1123,10 +1107,6 @@ DECL_HANDLER(read_directory_changes)
     if (dir->signaled>0)
         dir->signaled--;
 
-    /* clear the event */
-    if (event)
-        reset_event( event );
-
     /* if there's already a change in the queue, send it */
     if (!list_empty( &dir->change_q ) &&
         !list_empty( &dir->change_records ))
