@@ -69,7 +69,7 @@ static unsigned int file_map_access( struct object *obj, unsigned int access );
 static void file_destroy( struct object *obj );
 
 static int file_get_poll_events( struct fd *fd );
-static int file_flush( struct fd *fd, struct event **event );
+static void file_flush( struct fd *fd, struct event **event );
 static enum server_fd_type file_get_info( struct fd *fd, int *flags );
 
 static const struct object_ops file_ops =
@@ -84,6 +84,7 @@ static const struct object_ops file_ops =
     file_get_fd,                  /* get_fd */
     file_map_access,              /* map_access */
     no_lookup_name,               /* lookup_name */
+    no_open_file,                 /* open_file */
     fd_close_handle,              /* close_handle */
     file_destroy                  /* destroy */
 };
@@ -226,16 +227,10 @@ static int file_get_poll_events( struct fd *fd )
     return events;
 }
 
-static int file_flush( struct fd *fd, struct event **event )
+static void file_flush( struct fd *fd, struct event **event )
 {
-    int ret = 0, unix_fd = get_unix_fd( fd );
-
-    if (unix_fd != -1)
-    {
-        ret = (fsync( unix_fd ) != -1);
-        if (!ret) file_set_error();
-    }
-    return ret;
+    int unix_fd = get_unix_fd( fd );
+    if (unix_fd != -1 && fsync( unix_fd ) == -1) file_set_error();
 }
 
 static enum server_fd_type file_get_info( struct fd *fd, int *flags )
@@ -319,6 +314,12 @@ int get_file_unix_fd( struct file *file )
     return get_unix_fd( file->fd );
 }
 
+struct file *grab_file_unless_removable( struct file *file )
+{
+    if (is_fd_removable( file->fd )) return NULL;
+    return (struct file *)grab_object( file );
+}
+
 /* extend a file beyond the current end of file */
 static int extend_file( struct file *file, file_pos_t new_size )
 {
