@@ -75,6 +75,7 @@ static const struct object_ops process_ops =
     no_get_fd,                   /* get_fd */
     process_map_access,          /* map_access */
     no_lookup_name,              /* lookup_name */
+    no_open_file,                /* open_file */
     no_close_handle,             /* close_handle */
     process_destroy              /* destroy */
 };
@@ -119,6 +120,7 @@ static const struct object_ops startup_info_ops =
     no_get_fd,                     /* get_fd */
     no_map_access,                 /* map_access */
     no_lookup_name,                /* lookup_name */
+    no_open_file,                  /* open_file */
     no_close_handle,               /* close_handle */
     startup_info_destroy           /* destroy */
 };
@@ -492,7 +494,7 @@ static struct process_dll *process_load_dll( struct process *process, struct fil
             free( dll );
             return NULL;
         }
-        if (file) dll->file = (struct file *)grab_object( file );
+        if (file) dll->file = grab_file_unless_removable( file );
         list_add_tail( &process->dlls, &dll->entry );
     }
     return dll;
