@@ -97,6 +97,7 @@ static const struct object_ops master_socket_ops =
     no_get_fd,                     /* get_fd */
     no_map_access,                 /* map_access */
     no_lookup_name,                /* lookup_name */
+    no_open_file,                  /* open_file */
     no_close_handle,               /* close_handle */
     master_socket_destroy          /* destroy */
 };
@@ -251,11 +252,8 @@ static void send_reply( union generic_reply *reply )
             return;
         }
     }
-    if (current->reply_data)
-    {
-        free( current->reply_data );
-        current->reply_data = NULL;
-    }
+    free( current->reply_data );
+    current->reply_data = NULL;
     return;
 
  error:
@@ -402,13 +400,13 @@ int receive_fd( struct process *process )
 
     if (!ret)
     {
-        kill_process( process, NULL, 0 );
+        kill_process( process, 0 );
     }
     else if (ret > 0)
     {
         fprintf( stderr, "Protocol error: process %04x: partial recvmsg %d for fd\n",
                  process->id, ret );
-        kill_process( process, NULL, 1 );
+        kill_process( process, 1 );
     }
     else
     {
@@ -416,7 +414,7 @@ int receive_fd( struct process *process )
         {
             fprintf( stderr, "Protocol error: process %04x: ", process->id );
             perror( "recvmsg" );
-            kill_process( process, NULL, 1 );
+            kill_process( process, 1 );
         }
     }
     return -1;
@@ -450,17 +448,17 @@ int send_client_fd( struct process *process, int fd, obj_handle_t handle )
     if (ret >= 0)
     {
         fprintf( stderr, "Protocol error: process %04x: partial sendmsg %d\n", process->id, ret );
-        kill_process( process, NULL, 1 );
+        kill_process( process, 1 );
     }
     else if (errno == EPIPE)
     {
-        kill_process( process, NULL, 0 );
+        kill_process( process, 0 );
     }
     else
     {
         fprintf( stderr, "Protocol error: process %04x: ", process->id );
         perror( "sendmsg" );
-        kill_process( process, NULL, 1 );
+        kill_process( process, 1 );
     }
     return -1;
 }
@@ -803,6 +801,9 @@ void open_master_socket(void)
 
     /* init startup time */
     gettimeofday( &server_start_time, NULL );
+
+    /* init the process tracing mechanism */
+    init_tracing_mechanism();
 }
 
 /* master socket timer expiration handler */
