@@ -38,6 +38,7 @@
 #include "object.h"
 #include "process.h"
 #include "thread.h"
+#include "request.h"
 
 #if defined(linux) && defined(__SIGRTMIN)
 /* the signal used by linuxthreads as exit signal for clone() threads */
@@ -70,6 +71,7 @@ static const struct object_ops handler_ops =
     no_get_fd,                /* get_fd */
     no_map_access,            /* map_access */
     no_lookup_name,           /* lookup_name */
+    no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
     handler_destroy           /* destroy */
 };
@@ -185,7 +187,7 @@ static void sigint_callback(void)
 {
     kill_all_processes( NULL, 1 );
     flush_registry();
-    exit(1);
+    shutdown_master_socket();
 }
 
 /* SIGHUP handler */
