@@ -96,7 +96,7 @@ static void sock_destroy( struct object *obj );
 static int sock_get_poll_events( struct fd *fd );
 static void sock_poll_event( struct fd *fd, int event );
 static enum server_fd_type sock_get_info( struct fd *fd, int *flags );
-static void sock_queue_async( struct fd *fd, void *apc, void *user, void *iosb, int type, int count );
+static void sock_queue_async( struct fd *fd, const async_data_t *data, int type, int count );
 static void sock_cancel_async( struct fd *fd );
 
 static int sock_get_error( int err );
@@ -114,6 +114,7 @@ static const struct object_ops sock_ops =
     sock_get_fd,                  /* get_fd */
     sock_map_access,              /* map_access */
     no_lookup_name,               /* lookup_name */
+    no_open_file,                 /* open_file */
     fd_close_handle,              /* close_handle */
     sock_destroy                  /* destroy */
 };
@@ -284,7 +285,7 @@ static void sock_wake_up( struct sock *sock, int pollev )
     }
 }
 
-inline static int sock_error( struct fd *fd )
+static inline int sock_error( struct fd *fd )
 {
     unsigned int optval = 0, optlen;
 
@@ -508,8 +509,7 @@ static enum server_fd_type sock_get_info( struct fd *fd, int *flags )
     return FD_TYPE_SOCKET;
 }
 
-static void sock_queue_async( struct fd *fd, void *apc, void *user, void *iosb,
-                              int type, int count )
+static void sock_queue_async( struct fd *fd, const async_data_t *data, int type, int count )
 {
     struct sock *sock = get_fd_user( fd );
     struct list *queue;
@@ -544,8 +544,8 @@ static void sock_queue_async( struct fd *fd, void *apc, void *user, void *iosb,
     }
     else
     {
-        if (!create_async( current, NULL, queue, apc, user, iosb ))
-            return;
+        if (!create_async( current, NULL, queue, data )) return;
+        set_error( STATUS_PENDING );
     }
 
     pollev = sock_reselect( sock );
