@@ -97,7 +97,7 @@ static int func_cmp( const void *func1, const void *func2 )
 }
 
 /* add a name to a name table */
-inline static void add_name( struct name_table *table, const char *name )
+static inline void add_name( struct name_table *table, const char *name )
 {
     if (table->count == table->size)
     {
@@ -109,7 +109,7 @@ inline static void add_name( struct name_table *table, const char *name )
 }
 
 /* remove a name from a name table */
-inline static void remove_name( struct name_table *table, unsigned int idx )
+static inline void remove_name( struct name_table *table, unsigned int idx )
 {
     assert( idx < table->count );
     free( table->names[idx] );
@@ -119,7 +119,7 @@ inline static void remove_name( struct name_table *table, unsigned int idx )
 }
 
 /* make a name table empty */
-inline static void empty_name_table( struct name_table *table )
+static inline void empty_name_table( struct name_table *table )
 {
     unsigned int i;
 
@@ -128,7 +128,7 @@ inline static void empty_name_table( struct name_table *table )
 }
 
 /* locate a name in a (sorted) list */
-inline static const char *find_name( const char *name, const struct name_table *table )
+static inline const char *find_name( const char *name, const struct name_table *table )
 {
     char **res = NULL;
 
@@ -137,13 +137,13 @@ inline static const char *find_name( const char *name, const struct name_table *
 }
 
 /* sort a name table */
-inline static void sort_names( struct name_table *table )
+static inline void sort_names( struct name_table *table )
 {
     if (table->count) qsort( table->names, table->count, sizeof(*table->names), name_cmp );
 }
 
 /* locate an export in a (sorted) export list */
-inline static ORDDEF *find_export( const char *name, ORDDEF **table, int size )
+static inline ORDDEF *find_export( const char *name, ORDDEF **table, int size )
 {
     ORDDEF func, *odp, **res = NULL;
 
@@ -168,7 +168,7 @@ static void free_imports( struct import *imp )
 /* check whether a given dll is imported in delayed mode */
 static int is_delayed_import( const char *name )
 {
-    int i;
+    unsigned int i;
 
     for (i = 0; i < delayed_imports.count; i++)
     {
@@ -468,7 +468,8 @@ static void check_undefined_exports( DLLSPEC *spec )
 static char *create_undef_symbols_file( DLLSPEC *spec )
 {
     char *as_file, *obj_file;
-    unsigned int i;
+    int i;
+    unsigned int j;
     FILE *f;
 
     as_file = get_temp_file_name( output_file_name, ".s" );
@@ -482,8 +483,8 @@ static char *create_undef_symbols_file( DLLSPEC *spec )
         if (odp->flags & FLAG_FORWARD) continue;
         fprintf( f, "\t%s %s\n", get_asm_ptr_keyword(), asm_name(odp->link_name) );
     }
-    for (i = 0; i < extra_ld_symbols.count; i++)
-        fprintf( f, "\t%s %s\n", get_asm_ptr_keyword(), asm_name(extra_ld_symbols.names[i]) );
+    for (j = 0; j < extra_ld_symbols.count; j++)
+        fprintf( f, "\t%s %s\n", get_asm_ptr_keyword(), asm_name(extra_ld_symbols.names[j]) );
     fclose( f );
 
     obj_file = get_temp_file_name( output_file_name, ".o" );
@@ -557,7 +558,8 @@ void read_undef_symbols( DLLSPEC *spec, char **argv )
 /* resolve the imports for a Win32 module */
 int resolve_imports( DLLSPEC *spec )
 {
-    unsigned int i, j, removed;
+    int i;
+    unsigned int j, removed;
     ORDDEF *odp;
 
     sort_names( &ignore_symbols );
