@@ -44,7 +44,7 @@ static void dump_mdmp_string(DWORD rva)
         printf("<<?>>");
 }
 
-static const MINIDUMP_DIRECTORY* get_mdmp_dir(const MINIDUMP_HEADER* hdr, int str_idx)
+static const MINIDUMP_DIRECTORY* get_mdmp_dir(const MINIDUMP_HEADER* hdr, unsigned int str_idx)
 {
     const MINIDUMP_DIRECTORY*   dir;
     unsigned int                i;
@@ -66,7 +66,7 @@ enum FileSig get_kind_mdmp(void)
     pdw = PRD(0, sizeof(DWORD));
     if (!pdw) {printf("Can't get main signature, aborting\n"); return SIG_UNKNOWN;}
 
-    if (*pdw == 0x444D /* "MDMP" */) return SIG_MDMP;
+    if (*pdw == 0x504D444D /* "MDMP" */) return SIG_MDMP;
     return SIG_UNKNOWN;
 }
 
